/////////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2017- Statoil ASA
// 
//  ResInsight is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  ResInsight is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.
// 
//  See the GNU General Public License at <http://www.gnu.org/licenses/gpl.html> 
//  for more details.
//
/////////////////////////////////////////////////////////////////////////////////

#include "RiuSummaryVectorDescriptionMap.h"


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
RiuSummaryVectorDescriptionMap* RiuSummaryVectorDescriptionMap::instance()
{
    static RiuSummaryVectorDescriptionMap* singleton = new RiuSummaryVectorDescriptionMap;
    return singleton;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
std::string RiuSummaryVectorDescriptionMap::fieldInfo(const std::string &vectorName)
{
    std::map<std::string, std::string>::iterator it = m_summaryToDescMap.find(vectorName);
    
    if (it != m_summaryToDescMap.end())
    {
        return it->second;
    }
    else if (vectorName.size() > 1)
    {
        std::string key = vectorName.substr(0, 2);

        it = m_summaryToDescMap.find(key);

        if (it != m_summaryToDescMap.end())
        {
            return it->second;
        }
    }

    return vectorName;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
RiuSummaryVectorDescriptionMap::RiuSummaryVectorDescriptionMap()
{
    populateFieldToInfoMap();
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void RiuSummaryVectorDescriptionMap::populateFieldToInfoMap()
{
    m_summaryToDescMap  =
    {
        { "FOPR",        "Oil Production Rate" },
        { "FOPRA",       "Oil Production Rate above GOC" },
        { "FOPRB",       "Oil Production Rate below GOC" },
        { "FOPTA",       "Oil Production Total above GOC" },
        { "FOPTB",       "Oil Production Total below GOC" },
        { "FOPR1",       "Oil Production Rate above GOC" },
        { "FOPR2",       "Oil Production Rate below GOC" },
        { "FOPT1",       "Oil Production Total above GOC" },
        { "FOPT2",       "Oil Production Total below GOC" },
        { "FOMR",        "Oil Mass Rate" },
        { "FOMT",        "Oil Mass Total" },
        { "FODN",        "Oil Density at Surface Conditions" },
        { "FOPRH",       "Oil Production Rate History" },
        { "FOPRT",       "Oil Production Rate Target/Limit" },
        { "FOPRF",       "Free Oil Production Rate" },
        { "FOPRS",       "Solution Oil Production Rate" },
        { "FOPT",        "Oil Production Total" },
        { "FOPTH",       "Oil Production Total History" },
        { "FOPTF",       "Free Oil Production Total" },
        { "FOPTS",       "Solution Oil Production Total" },
        { "FOIR",        "Oil Injection Rate" },
        { "FOIRH",       "Oil Injection Rate History" },
        { "FOIRT",       "Oil Injection Rate Target/Limit" },
        { "FOIT",        "Oil Injection Total" },
        { "FOITH",       "Oil Injection Total History" },
        { "FOPP",        "Oil Potential Production rate" },
        { "FOPP2",       "Oil Potential Production rate" },
        { "FOPI",        "Oil Potential Injection rate" },
        { "FOPI2",       "Oil Potential Injection rate" },
        { "FOVPR",       "Oil Voidage Production Rate" },
        { "FOVPT",       "Oil Voidage Production Total" },
        { "FOVIR",       "Oil Voidage Injection Rate" },
        { "FOVIT",       "Oil Voidage Injection Total" },
        { "FOnPR",       "nth separator stage oil rate" },
        { "FOnPT",       "nth separator stage oil total" },
        { "FEOR",        "Export Oil Rate" },
        { "FEOT",        "Export Oil Total" },
        { "FEOMF",       "Export Oil Mole Fraction" },
        { "FWPR",        "Water Production Rate" },
        { "FWMR",        "Water Mass Rate" },
        { "FWMT",        "Water Mass Total" },
        { "FWPRH",       "Water Production Rate History" },
        { "FWPRT",       "Water Production Rate Target/Limit" },
        { "FWPT",        "Water Production Total" },
        { "FWPTH",       "Water Production Total History" },
        { "FWIR",        "Water Injection Rate" },
        { "FWIRH",       "Water Injection Rate History" },
        { "FWIRT",       "Water Injection Rate Target/Limit" },
        { "FWIT",        "Water Injection Total" },
        { "FWITH",       "Water Injection Total History" },
        { "FWPP",        "Water Potential Production rate" },
        { "FWPP2",       "Water Potential Production rate" },
        { "FWPI",        "Water Potential Injection rate" },
        { "FWPI2",       "Water Potential Injection rate" },
        { "FWVPR",       "Water Voidage Production Rate" },
        { "FWVPT",       "Water Voidage Production Total" },
        { "FWVIR",       "Water Voidage Injection Rate" },
        { "FWVIT",       "Water Voidage Injection Total" },
        { "FWPIR",       "Ratio of produced water to injected water (percentage)" },
        { "FWMPR",       "Water component molar production rate" },
        { "FWMPT",       "Water component molar production total" },
        { "FWMIR",       "Water component molar injection rate" },
        { "FWMIT",       "Water component molar injection total" },
        { "FGPR",        "Gas Production Rate" },
        { "FGPRA",       "Gas Production Rate above" },
        { "FGPRB",       "Gas Production Rate below" },
        { "FGPTA",       "Gas Production Total above" },
        { "FGPTB",       "Gas Production Total below" },
        { "FGPR1",       "Gas Production Rate above GOC" },
        { "FGPR2",       "Gas Production Rate below GOC" },
        { "FGPT1",       "Gas Production Total above GOC" },
        { "FGPT2",       "Gas Production Total below GOC" },
        { "FGMR",        "Gas Mass Rate" },
        { "FGMT",        "Gas Mass Total" },
        { "FGDN",        "Gas Density at Surface Conditions" },
        { "FGPRH",       "Gas Production Rate History" },
        { "FGPRT",       "Gas Production Rate Target/Limit" },
        { "FGPRF",       "Free Gas Production Rate" },
        { "FGPRS",       "Solution Gas Production Rate" },
        { "FGPT",        "Gas Production Total" },
        { "FGPTH",       "Gas Production Total History" },
        { "FGPTF",       "Free Gas Production Total" },
        { "FGPTS",       "Solution Gas Production Total" },
        { "FGIR",        "Gas Injection Rate" },
        { "FGIRH",       "Gas Injection Rate History" },
        { "FGIRT",       "Gas Injection Rate Target/Limit" },
        { "FGIT",        "Gas Injection Total" },
        { "FGITH",       "Gas Injection Total History" },
        { "FGPP",        "Gas Potential Production rate" },
        { "FGPP2",       "Gas Potential Production rate" },
        { "FGPPS",       "Solution" },
        { "FGPPS2",      "Solution" },
        { "FGPPF",       "Free Gas Potential Production rate" },
        { "FGPPF2",      "Free Gas Potential Production rate" },
        { "FGPI",        "Gas Potential Injection rate" },
        { "FGPI2",       "Gas Potential Injection rate" },
        { "FSGR",        "Sales Gas Rate" },
        { "FGSR",        "Sales Gas Rate" },
        { "FSGT",        "Sales Gas Total" },
        { "FGST",        "Sales Gas Total" },
        { "FSMF",        "Sales Gas Mole Fraction" },
        { "FFGR",        "Fuel Gas Rate, at and below this group" },
        { "FFGT",        "Fuel Gas cumulative Total, at and below this group" },
        { "FFMF",        "Fuel Gas Mole Fraction" },
        { "FGCR",        "Gas Consumption Rate, at and below this group" },
        { "FGCT",        "Gas Consumption Total, at and below this group" },
        { "FGIMR",       "Gas Import Rate, at and below this group" },
        { "FGIMT",       "Gas Import Total, at and below this group" },
        { "FGLIR",       "Gas Lift Injection Rate" },
        { "FWGPR",       "Wet Gas Production Rate" },
        { "FWGPT",       "Wet Gas Production Total" },
        { "FWGPRH",      "Wet Gas Production Rate History" },
        { "FWGIR",       "Wet Gas Injection Rate" },
        { "FWGIT",       "Wet Gas Injection Total" },
        { "FEGR",        "Export Gas Rate" },
        { "FEGT",        "Export Gas Total" },
        { "FEMF",        "Export Gas Mole Fraction" },
        { "FEXGR",       "Excess Gas Rate" },
        { "FEXGT",       "Excess Gas Total" },
        { "FRGR",        "Re-injection Gas Rate" },
        { "FRGT",        "Re-injection Gas Total" },
        { "FGnPR",       "nth separator stage gas rate" },
        { "FGnPT",       "nth separator stage gas total" },
        { "FGVPR",       "Gas Voidage Production Rate" },
        { "FGVPT",       "Gas Voidage Production Total" },
        { "FGVIR",       "Gas Voidage Injection Rate" },
        { "FGVIT",       "Gas Voidage Injection Total" },
        { "FGQ",         "Gas Quality" },
        { "FLPR",        "Liquid Production Rate" },
        { "FLPRH",       "Liquid Production Rate History" },
        { "FLPRT",       "Liquid Production Rate Target/Limit" },
        { "FLPT",        "Liquid Production Total" },
        { "FLPTH",       "Liquid Production Total History" },
        { "FVPR",        "Res Volume Production Rate" },
        { "FVPRT",       "Res Volume Production Rate Target/Limit" },
        { "FVPT",        "Res Volume Production Total" },
        { "FVIR",        "Res Volume Injection Rate" },
        { "FVIRT",       "Res Volume Injection Rate Target/Limit" },
        { "FVIT",        "Res Volume Injection Total" },
        { "FWCT",        "Water Cut" },
        { "FWCTH",       "Water Cut History" },
        { "FGOR",        "Gas-Oil Ratio" },
        { "FGORH",       "Gas-Oil Ratio History" },
        { "FOGR",        "Oil-Gas Ratio" },
        { "FOGRH",       "Oil-Gas Ratio History" },
        { "FWGR",        "Water-Gas Ratio" },
        { "FWGRH",       "Water-Gas Ratio History" },
        { "FGLR",        "Gas-Liquid Ratio" },
        { "FGLRH",       "Gas-Liquid Ratio History" },
        { "FMCTP",       "Mode of Control for group Production" },
        { "FMCTW",       "Mode of Control for group Water Injection" },
        { "FMCTG",       "Mode of Control for group Gas Injection" },
        { "FMWPT",       "Total number of production wells" },
        { "FMWPR",       "Number of production wells currently flowing" },
        { "FMWPA",       "Number of abandoned production wells" },
        { "FMWPU",       "Number of unused production wells" },
        { "FMWPG",       "Number of producers on group control" },
        { "FMWPO",       "Number of producers controlled by own oil rate limit" },
        { "FMWPS",       "Number of producers on own surface rate limit control" },
        { "FMWPV",       "Number of producers on own reservoir volume rate limit control" },
        { "FMWPP",       "Number of producers on pressure control" },
        { "FMWPL",       "Number of producers using artificial lift" },
        { "FMWIT",       "Total number of injection wells" },
        { "FMWIN",       "Number of injection wells currently flowing" },
        { "FMWIA",       "Number of abandoned injection wells" },
        { "FMWIU",       "Number of unused injection wells" },
        { "FMWIG",       "Number of injectors on group control" },
        { "FMWIS",       "Number of injectors on own surface rate limit control" },
        { "FMWIV",       "Number of injectors on own reservoir volume rate limit control" },
        { "FMWIP",       "Number of injectors on pressure control" },
        { "FMWDR",       "Number of drilling events this timestep" },
        { "FMWDT",       "Number of drilling events in total" },
        { "FMWWO",       "Number of workover events this timestep" },
        { "FMWWT",       "Number of workover events in total" },
        { "FEPR",        "Energy Production Rate" },
        { "FEPT",        "Energy Production Total" },
        { "FNLPR",       "NGL Production Rate" },
        { "FNLPT",       "NGL Production Total" },
        { "FNLPRH",      "NGL Production Rate History" },
        { "FNLPTH",      "NGL Production Total History" },
        { "FAMF",        "Component aqueous mole fraction, from producing completions" },
        { "FXMF",        "Liquid Mole Fraction" },
        { "FYMF",        "Vapor Mole Fraction" },
        { "FXMFn",       "Liquid Mole Fraction for nth separator stage" },
        { "FYMFn",       "Vapor Mole Fraction for nth separator stage" },
        { "FZMF",        "Total Mole Fraction" },
        { "FCMPR",       "Hydrocarbon Component Molar Production Rates" },
        { "FCMPT",       "Hydrocarbon Component" },
        { "FCMIR",       "Hydrocarbon Component Molar Injection Rates" },
        { "FCMIT",       "Hydrocarbon Component Molar Injection Totals" },
        { "FHMIR",       "Hydrocarbon Molar Injection Rate" },
        { "FHMIT",       "Hydrocarbon Molar Injection Total" },
        { "FHMPR",       "Hydrocarbon Molar Production Rate" },
        { "FHMPT",       "Hydrocarbon Molar Production Total" },
        { "FCHMR",       "Hydrocarbon Component" },
        { "FCHMT",       "Hydrocarbon Component" },
        { "FCWGPR",      "Hydrocarbon Component Wet Gas Production Rate" },
        { "FCWGPT",      "Hydrocarbon Component Wet Gas Production Total" },
        { "FCWGIR",      "Hydrocarbon Component Wet Gas Injection Rate" },
        { "FCWGIT",      "Hydrocarbon Component Wet Gas Injection Total" },
        { "FCGMR",       "Hydrocarbon component" },
        { "FCGMT",       "Hydrocarbon component" },
        { "FCOMR",       "Hydrocarbon component" },
        { "FCOMT",       "Hydrocarbon component" },
        { "FCNMR",       "Hydrocarbon component molar rates in the NGL phase" },
        { "FCNWR",       "Hydrocarbon component mass rates in the NGL phase" },
        { "FCGMRn",      "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "FCGRn",       "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "FCOMRn",      "Hydrocarbon component" },
        { "FCORn",       "Hydrocarbon component" },
        { "FMUF",        "Make-up fraction" },
        { "FAMR",        "Make-up gas rate" },
        { "FAMT",        "Make-up gas total" },
        { "FGSPR",       "Target sustainable rate for most recent sustainable capacity test for gas" },
        { "FGSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for gas" },
        { "FGSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for gas" },
        { "FGSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for gas" },
        { "FGSTP",       "Test period for the most recent sustainable capacity test for gas" },
        { "FOSPR",       "Target sustainable rate for most recent sustainable capacity test for oil" },
        { "FOSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for oil" },
        { "FOSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for oil" },
        { "FOSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for oil" },
        { "FOSTP",       "Test period for the most recent sustainable capacity test for oil" },
        { "FWSPR",       "Target sustainable rate for most recent sustainable capacity test for water" },
        { "FWSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for water" },
        { "FWSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for water" },
        { "FWSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for water" },
        { "FWSTP",       "Test period for the most recent sustainable capacity test for water" },
        { "FGPRG",       "Gas production rate" },
        { "FOPRG",       "Oil production rate" },
        { "FNLPRG",      "NGL production rate" },
        { "FXMFG",       "Liquid mole fraction" },
        { "FYMFG",       "Vapor mole fraction" },
        { "FCOMRG",      "Hydrocarbon component" },
        { "FCGMRG",      "Hydrocarbon component molar rates in the gas phase" },
        { "FCNMRG",      "Hydrocarbon component molar rates in the NGL phase" },
        { "FPR",         "Pressure average value" },
        { "FPRH",        "Pressure average value" },
        { "FPRP",        "Pressure average value" },
        { "FPRGZ",       "P/Z" },
        { "FRS",         "Gas-oil ratio" },
        { "FRV",         "Oil-gas ratio" },
        { "FCHIP",       "Component Hydrocarbon as Wet Gas" },
        { "FCMIP",       "Component Hydrocarbon as Moles" },
        { "FPPC",        "Initial Contact Corrected Potential" },
        { "FREAC",       "Reaction rate. The reaction number is given as a component index" },
        { "FREAT",       "Reaction total. The reaction number is given as a component index" },
        { "FRPV",        "Pore Volume at Reservoir conditions" },
        { "FOPV",        "Pore Volume containing Oil" },
        { "FWPV",        "Pore Volume containing Water" },
        { "FGPV",        "Pore Volume containing Gas" },
        { "FHPV",        "Pore Volume containing Hydrocarbon" },
        { "FRTM",        "Transmissibility Multiplier associated with rock compaction" },
        { "FOE",         "(OIP(initial - OIP(now) / OIP(initial)" },
        { "FOEW",        "Oil Production from Wells / OIP(initial)" },
        { "FOEIW",       "(OIP(initial - OIP(now) / Initial Mobile Oil with respect to Water" },
        { "FOEWW",       "Oil Production from Wells / Initial Mobile Oil with respect to Water" },
        { "FOEIG",       "(OIP(initial - OIP(now) / Initial Mobile Oil with respect to Gas" },
        { "FOEWG",       "Oil Production from Wells / Initial Mobile Oil with respect to Gas" },
        { "FORMR",       "Total stock tank oil produced by rock compaction" },
        { "FORMW",       "Total stock tank oil produced by water influx" },
        { "FORMG",       "Total stock tank oil produced by gas influx" },
        { "FORME",       "Total stock tank oil produced by oil expansion" },
        { "FORMS",       "Total stock tank oil produced by solution gas" },
        { "FORMF",       "Total stock tank oil produced by free gas influx" },
        { "FORMX",       "Total stock tank oil produced by 'traced' water influx" },
        { "FORMY",       "Total stock tank oil produced by other water influx" },
        { "FORFR",       "Fraction of total oil produced by rock compaction" },
        { "FORFW",       "Fraction of total oil produced by water influx" },
        { "FORFG",       "Fraction of total oil produced by gas influx" },
        { "FORFE",       "Fraction of total oil produced by oil expansion" },
        { "FORFS",       "Fraction of total oil produced by solution gas" },
        { "FORFF",       "Fraction of total oil produced by free gas influx" },
        { "FORFX",       "Fraction of total oil produced by 'traced' water influx" },
        { "FORFY",       "Fraction of total oil produced by other water influx" },
        { "FAQR",        "Aquifer influx rate" },
        { "FAQT",        "Cumulative aquifer influx" },
        { "FAQRG",       "Aquifer influx rate" },
        { "FAQTG",       "Cumulative aquifer influx" },
        { "FAQER",       "Aquifer thermal energy influx rate" },
        { "FAQET",       "Cumulative aquifer thermal energy influx" },
        { "FNQR",        "Aquifer influx rate" },
        { "FNQT",        "Cumulative aquifer influx" },
        { "FTPR",        "Tracer Production Rate" },
        { "FTPT",        "Tracer Production Total" },
        { "FTPC",        "Tracer Production Concentration" },
        { "FTIR",        "Tracer Injection Rate" },
        { "FTIT",        "Tracer Injection Total" },
        { "FTIC",        "Tracer Injection Concentration" },
        { "FTMR",        "Traced mass Rate" },
        { "FTMT",        "Traced mass Total" },
        { "FTQR",        "Traced molar Rate" },
        { "FTCM",        "Tracer Carrier molar Rate" },
        { "FTMF",        "Traced molar fraction" },
        { "FTVL",        "Traced liquid volume rate" },
        { "FTVV",        "Traced vapor volume rate" },
        { "FTTL",        "Traced liquid volume total" },
        { "FTTV",        "Traced vapor volume total" },
        { "FTML",        "Traced mass liquid rate" },
        { "FTMV",        "Traced mass vapor rate" },
        { "FTLM",        "Traced mass liquid total" },
        { "FTVM",        "Traced mass vapor total" },
        { "FTIPT",       "Tracer In Place" },
        { "FTIPF",       "Tracer In Place" },
        { "FTIPS",       "Tracer In Place" },
        { "FAPI",        "Oil API" },
        { "FSPR",        "Salt Production Rate" },
        { "FSPT",        "Salt Production Total" },
        { "FSIR",        "Salt Injection Rate" },
        { "FSIT",        "Salt Injection Total" },
        { "FSPC",        "Salt Production Concentration" },
        { "FSIC",        "Salt Injection Concentration" },
        { "FSIP",        "Salt In Place" },
        { "GTPRANI",     "Anion Production Rate" },
        { "GTPTANI",     "Anion Production Total" },
        { "GTIRANI",     "Anion Injection Rate" },
        { "GTITANI",     "Anion Injection Total" },
        { "GTPRCAT",     "Cation Production Rate" },
        { "GTPTCAT",     "Cation Production Total" },
        { "GTIRCAT",     "Cation Injection Rate" },
        { "GTITCAT",     "Cation Injection Total" },
        { "FTPCHEA",     "Production Temperature" },
        { "FTICHEA",     "Injection Temperature" },
        { "FTPRHEA",     "Energy flows" },
        { "FTPTHEA",     "Energy Production Total" },
        { "FTIRHEA",     "Energy flows" },
        { "FTITHEA",     "Energy Injection Total" },
        { "FTIPTHEA",    "Difference in Energy in place between current and initial time" },
        { "FTPR",        "Tracer Production Rate" },
        { "FTPT",        "Tracer Production Total" },
        { "FTPC",        "Tracer Production Concentration" },
        { "FTIR",        "Tracer Injection Rate" },
        { "FTIT",        "Tracer Injection Total" },
        { "FTIC",        "Tracer Injection Concentration" },
        { "FTIPT",       "Tracer In Place" },
        { "FTIPF",       "Tracer In Place" },
        { "FTIPS",       "Tracer In Place" },
        { "FTIP#",       " Tracer In Place in phase # (1,2,3,...)" },
        { "FTADS",       "Tracer Adsorption total" },
        { "FTDCY",       "Decayed tracer" },
        { "FTIRF",       "Tracer Injection Rate" },
        { "FTIRS",       "Tracer Injection Rate" },
        { "FTPRF",       "Tracer Production Rate" },
        { "FTPRS",       "Tracer Production Rate" },
        { "FTITF",       "Tracer Injection Total" },
        { "FTITS",       "Tracer Injection Total" },
        { "FTPTF",       "Tracer Production Total" },
        { "FTPTS",       "Tracer Production Total" },
        { "FTICF",       "Tracer Injection Concentration" },
        { "FTICS",       "Tracer Injection Concentration" },
        { "FTPCF",       "Tracer Production" },
        { "FTPCS",       "Tracer Production" },
        { "FMPR",        "Methane Production Rate" },
        { "FMPT",        "Methane Production Total" },
        { "FMIR",        "Methane Injection Rate" },
        { "FMIT",        "Methane Injection Total" },
        { "FCGC",        "Bulk Coal Gas Concentration" },
        { "FCSC",        "Bulk Coal Solvent Concentration" },
        { "FTPRFOA",     "Production Rate" },
        { "FTPTFOA",     "Production Total" },
        { "FTIRFOA",     "Injection Rate" },
        { "FTITFOA",     "Injection Total" },
        { "FTIPTFOA",    "In Solution" },
        { "FTADSFOA",    "Adsorption total" },
        { "FTDCYFOA",    "Decayed tracer" },
        { "FTMOBFOA",    "Gas mobility factor" },
        { "FTPRFOA",     "Production Rate" },
        { "FTPTFOA",     "Production Total" },
        { "FTIRFOA",     "Injection Rate" },
        { "FTITFOA",     "Injection Total" },
        { "FTIPTFOA",    "In Solution" },
        { "FTADSFOA",    "Adsorption total" },
        { "FTDCYFOA",    "Decayed tracer" },
        { "FTMOBFOA",    "Gas mobility factor" },
        { "FSGR",        "Sales Gas Rate" },
        { "FGSR",        "Sales Gas Rate" },
        { "FSGT",        "Sales Gas Total" },
        { "FGST",        "Sales Gas Total" },
        { "FGDC",        "Gas Delivery Capacity" },
        { "FGDCQ",       "Field/Group Gas DCQ" },
        { "FGCR",        "Gas consumption rate, at and below this group" },
        { "FGCT",        "Gas consumption cumulative total, at and below this group" },
        { "FFGR",        "Fuel Gas rate, at and below this group" },
        { "FFGT",        "Fuel Gas cumulative total, at and below this group" },
        { "FGIMR",       "Gas import rate, at and below this group" },
        { "FGIMT",       "Gas import cumulative total, at and below this group" },
        { "FGLIR",       "Gas Lift Injection Rate" },
        { "FGCV",        "Gas Calorific Value" },
        { "FGQ",         "Gas molar Quality" },
        { "FEPR",        "Energy Production Rate" },
        { "FEPT",        "Energy Production Total" },
        { "FESR",        "Energy Sales Rate" },
        { "FEST",        "Energy Sales Total" },
        { "FEDC",        "Energy Delivery Capacity" },
        { "FEDCQ",       "Energy DCQ" },
        { "FCPR",        "Polymer Production Rate" },
        { "FCPC",        "Polymer Production Concentration" },
        { "FCPT",        "Polymer Production Total" },
        { "FCIR",        "Polymer Injection Rate" },
        { "FCIC",        "Polymer Injection Concentration" },
        { "FCIT",        "Polymer Injection Total" },
        { "FCIP",        "Polymer In Solution" },
        { "FCAD",        "Polymer Adsorption total" },
        { "FSPR",        "Salt Production Rate" },
        { "FSPT",        "Salt Production Total" },
        { "FSIR",        "Salt Injection Rate" },
        { "FSIT",        "Salt Injection Total" },
        { "FSIP",        "Salt In Place" },
        { "PSSPR",       "Log of the pressure change per unit time" },
        { "PSSSO",       "Log of the oil saturation change per unit time" },
        { "PSSSW",       "Log of the water saturation change per unit time" },
        { "PSSSG",       "Log of the gas saturation change per unit time" },
        { "PSSSC",       "Log of the salt concentration change per unit time" },
        { "FNPR",        "Solvent Production Rate" },
        { "FNPT",        "Solvent Production Total" },
        { "FNIR",        "Solvent Injection Rate" },
        { "FNIT",        "Solvent Injection Total" },
        { "FNIP",        "Solvent In Place" },
        { "FTPRSUR",     "Production Rate" },
        { "FTPTSUR",     "Production Total" },
        { "FTIRSUR",     "Injection Rate" },
        { "FTITSUR",     "Injection Total" },
        { "FTIPTSUR",    "In Solution" },
        { "FTADSUR",     "Adsorption total" },
        { "FTPRALK",     "Production Rate" },
        { "FTPTALK",     "Production Total" },
        { "FTIRALK",     "Injection Rate" },
        { "FTITALK",     "Injection Total" },
        { "FU",          "User-defined field quantity" },

        { "GOPR",        "Oil Production Rate" },
        { "GOPRA",       "Oil Production Rate above GOC" },
        { "GOPRB",       "Oil Production Rate below GOC" },
        { "GOPTA",       "Oil Production Total above GOC" },
        { "GOPTB",       "Oil Production Total below GOC" },
        { "GOPR1",       "Oil Production Rate above GOC" },
        { "GOPR2",       "Oil Production Rate below GOC" },
        { "GOPT1",       "Oil Production Total above GOC" },
        { "GOPT2",       "Oil Production Total below GOC" },
        { "GOMR",        "Oil Mass Rate" },
        { "GOMT",        "Oil Mass Total" },
        { "GODN",        "Oil Density at Surface Conditions" },
        { "GOPRH",       "Oil Production Rate History" },
        { "GOPRT",       "Oil Production Rate Target/Limit" },
        { "GOPRL",       "Oil Production Rate Target/Limit" },
        { "GOPRF",       "Free Oil Production Rate" },
        { "GOPRS",       "Solution Oil Production Rate" },
        { "GOPT",        "Oil Production Total" },
        { "GOPTH",       "Oil Production Total History" },
        { "GOPTF",       "Free Oil Production Total" },
        { "GOPTS",       "Solution Oil Production Total" },
        { "GOIR",        "Oil Injection Rate" },
        { "GOIRH",       "Oil Injection Rate History" },
        { "GOIRT",       "Oil Injection Rate Target/Limit" },
        { "GOIRL",       "Oil Injection Rate Target/Limit" },
        { "GOIT",        "Oil Injection Total" },
        { "GOITH",       "Oil Injection Total History" },
        { "GOPP",        "Oil Potential Production rate" },
        { "GOPP2",       "Oil Potential Production rate" },
        { "GOPI",        "Oil Potential Injection rate" },
        { "GOPI2",       "Oil Potential Injection rate" },
        { "GOPGR",       "Oil Production Guide Rate" },
        { "GOIGR",       "Oil Injection Guide Rate" },
        { "GOVPR",       "Oil Voidage Production Rate" },
        { "GOVPT",       "Oil Voidage Production Total" },
        { "GOVIR",       "Oil Voidage Injection Rate" },
        { "GOVIT",       "Oil Voidage Injection Total" },
        { "GOnPR",       "nth separator stage oil rate" },
        { "GOnPT",       "nth separator stage oil total" },
        { "GEOR",        "Export Oil Rate" },
        { "GEOT",        "Export Oil Total" },
        { "GEOMF",       "Export Oil Mole Fraction" },
        { "GWPR",        "Water Production Rate" },
        { "GWMR",        "Water Mass Rate" },
        { "GWMT",        "Water Mass Total" },
        { "GWPRH",       "Water Production Rate History" },
        { "GWPRT",       "Water Production Rate Target/Limit" },
        { "GWPRL",       "Water Production Rate Target/Limit" },
        { "GWPT",        "Water Production Total" },
        { "GWPTH",       "Water Production Total History" },
        { "GWIR",        "Water Injection Rate" },
        { "GWIRH",       "Water Injection Rate History" },
        { "GWIRT",       "Water Injection Rate Target/Limit" },
        { "GWIRL",       "Water Injection Rate Target/Limit" },
        { "GWIT",        "Water Injection Total" },
        { "GWITH",       "Water Injection Total History" },
        { "GWPP",        "Water Potential Production rate" },
        { "GWPP2",       "Water Potential Production rate" },
        { "GWPI",        "Water Potential Injection rate" },
        { "GWPI2",       "Water Potential Injection rate" },
        { "GWPGR",       "Water Production Guide Rate" },
        { "GWIGR",       "Water Injection Guide Rate" },
        { "GWVPR",       "Water Voidage Production Rate" },
        { "GWVPT",       "Water Voidage Production Total" },
        { "GWVIR",       "Water Voidage Injection Rate" },
        { "GWVIT",       "Water Voidage Injection Total" },
        { "GWPIR",       "Ratio of produced water to injected water (percentage)" },
        { "GWMPR",       "Water component molar production rate" },
        { "GWMPT",       "Water component molar production total" },
        { "GWMIR",       "Water component molar injection rate" },
        { "GWMIT",       "Water component molar injection total" },
        { "GGPR",        "Gas Production Rate" },
        { "GGPRA",       "Gas Production Rate above" },
        { "GGPRB",       "Gas Production Rate below" },
        { "GGPTA",       "Gas Production Total above" },
        { "GGPTB",       "Gas Production Total below" },
        { "GGPR1",       "Gas Production Rate above GOC" },
        { "GGPR2",       "Gas Production Rate below GOC" },
        { "GGPT1",       "Gas Production Total above GOC" },
        { "GGPT2",       "Gas Production Total below GOC" },
        { "GGMR",        "Gas Mass Rate" },
        { "GGMT",        "Gas Mass Total" },
        { "GGDN",        "Gas Density at Surface Conditions" },
        { "GGPRH",       "Gas Production Rate History" },
        { "GGPRT",       "Gas Production Rate Target/Limit" },
        { "GGPRL",       "Gas Production Rate Target/Limit" },
        { "GGPRF",       "Free Gas Production Rate" },
        { "GGPRS",       "Solution Gas Production Rate" },
        { "GGPT",        "Gas Production Total" },
        { "GGPTH",       "Gas Production Total History" },
        { "GGPTF",       "Free Gas Production Total" },
        { "GGPTS",       "Solution Gas Production Total" },
        { "GGIR",        "Gas Injection Rate" },
        { "GGIRH",       "Gas Injection Rate History" },
        { "GGIRT",       "Gas Injection Rate Target/Limit" },
        { "GGIRL",       "Gas Injection Rate Target/Limit" },
        { "GGIT",        "Gas Injection Total" },
        { "GGITH",       "Gas Injection Total History" },
        { "GGPP",        "Gas Potential Production rate" },
        { "GGPP2",       "Gas Potential Production rate" },
        { "GGPPS",       "Solution" },
        { "GGPPS2",      "Solution" },
        { "GGPPF",       "Free Gas Potential Production rate" },
        { "GGPPF2",      "Free Gas Potential Production rate" },
        { "GGPI",        "Gas Potential Injection rate" },
        { "GGPI2",       "Gas Potential Injection rate" },
        { "GGPGR",       "Gas Production Guide Rate" },
        { "GGIGR",       "Gas Injection Guide Rate" },
        { "GSGR",        "Sales Gas Rate" },
        { "GGSR",        "Sales Gas Rate" },
        { "GSGT",        "Sales Gas Total" },
        { "GGST",        "Sales Gas Total" },
        { "GSMF",        "Sales Gas Mole Fraction" },
        { "GFGR",        "Fuel Gas Rate, at and below this group" },
        { "GFGT",        "Fuel Gas cumulative Total, at and below this group" },
        { "GFMF",        "Fuel Gas Mole Fraction" },
        { "GGCR",        "Gas Consumption Rate, at and below this group" },
        { "GGCT",        "Gas Consumption Total, at and below this group" },
        { "GGIMR",       "Gas Import Rate, at and below this group" },
        { "GGIMT",       "Gas Import Total, at and below this group" },
        { "GGLIR",       "Gas Lift Injection Rate" },
        { "GWGPR",       "Wet Gas Production Rate" },
        { "GWGPT",       "Wet Gas Production Total" },
        { "GWGPRH",      "Wet Gas Production Rate History" },
        { "GWGIR",       "Wet Gas Injection Rate" },
        { "GWGIT",       "Wet Gas Injection Total" },
        { "GEGR",        "Export Gas Rate" },
        { "GEGT",        "Export Gas Total" },
        { "GEMF",        "Export Gas Mole Fraction" },
        { "GEXGR",       "Excess Gas Rate" },
        { "GEXGT",       "Excess Gas Total" },
        { "GRGR",        "Re-injection Gas Rate" },
        { "GRGT",        "Re-injection Gas Total" },
        { "GGnPR",       "nth separator stage gas rate" },
        { "GGnPT",       "nth separator stage gas total" },
        { "GGVPR",       "Gas Voidage Production Rate" },
        { "GGVPT",       "Gas Voidage Production Total" },
        { "GGVIR",       "Gas Voidage Injection Rate" },
        { "GGVIT",       "Gas Voidage Injection Total" },
        { "GGQ",         "Gas Quality" },
        { "GLPR",        "Liquid Production Rate" },
        { "GLPRH",       "Liquid Production Rate History" },
        { "GLPRT",       "Liquid Production Rate Target/Limit" },
        { "GLPRL",       "Liquid Production Rate Target/Limit" },
        { "GLPT",        "Liquid Production Total" },
        { "GLPTH",       "Liquid Production Total History" },
        { "GVPR",        "Res Volume Production Rate" },
        { "GVPRT",       "Res Volume Production Rate Target/Limit" },
        { "GVPRL",       "Res Volume Production Rate Target/Limit" },
        { "GVPT",        "Res Volume Production Total" },
        { "GVPGR",       "Res Volume Production Guide Rate" },
        { "GVIR",        "Res Volume Injection Rate" },
        { "GVIRT",       "Res Volume Injection Rate Target/Limit" },
        { "GVIRL",       "Res Volume Injection Rate Target/Limit" },
        { "GVIT",        "Res Volume Injection Total" },
        { "GWCT",        "Water Cut" },
        { "GWCTH",       "Water Cut History" },
        { "GGOR",        "Gas-Oil Ratio" },
        { "GGORH",       "Gas-Oil Ratio History" },
        { "GOGR",        "Oil-Gas Ratio" },
        { "GOGRH",       "Oil-Gas Ratio History" },
        { "GWGR",        "Water-Gas Ratio" },
        { "GWGRH",       "Water-Gas Ratio History" },
        { "GGLR",        "Gas-Liquid Ratio" },
        { "GGLRH",       "Gas-Liquid Ratio History" },
        { "GMCTP",       "Mode of Control for group Production" },
        { "GMCTW",       "Mode of Control for group Water Injection" },
        { "GMCTG",       "Mode of Control for group Gas Injection" },
        { "GMWPT",       "Total number of production wells" },
        { "GMWPR",       "Number of production wells currently flowing" },
        { "GMWPA",       "Number of abandoned production wells" },
        { "GMWPU",       "Number of unused production wells" },
        { "GMWPG",       "Number of producers on group control" },
        { "GMWPO",       "Number of producers controlled by own oil rate limit" },
        { "GMWPS",       "Number of producers on own surface rate limit control" },
        { "GMWPV",       "Number of producers on own reservoir volume rate limit control" },
        { "GMWPP",       "Number of producers on pressure control" },
        { "GMWPL",       "Number of producers using artificial lift" },
        { "GMWIT",       "Total number of injection wells" },
        { "GMWIN",       "Number of injection wells currently flowing" },
        { "GMWIA",       "Number of abandoned injection wells" },
        { "GMWIU",       "Number of unused injection wells" },
        { "GMWIG",       "Number of injectors on group control" },
        { "GMWIS",       "Number of injectors on own surface rate limit control" },
        { "GMWIV",       "Number of injectors on own reservoir volume rate limit control" },
        { "GMWIP",       "Number of injectors on pressure control" },
        { "GMWDR",       "Number of drilling events this timestep" },
        { "GMWDT",       "Number of drilling events in total" },
        { "GMWWO",       "Number of workover events this timestep" },
        { "GMWWT",       "Number of workover events in total" },
        { "GEPR",        "Energy Production Rate" },
        { "GEPT",        "Energy Production Total" },
        { "GEFF",        "Efficiency Factor" },
        { "GNLPR",       "NGL Production Rate" },
        { "GNLPT",       "NGL Production Total" },
        { "GNLPRH",      "NGL Production Rate History" },
        { "GNLPTH",      "NGL Production Total History" },
        { "GAMF",        "Component aqueous mole fraction, from producing completions" },
        { "GXMF",        "Liquid Mole Fraction" },
        { "GYMF",        "Vapor Mole Fraction" },
        { "GXMFn",       "Liquid Mole Fraction for nth separator stage" },
        { "GYMFn",       "Vapor Mole Fraction for nth separator stage" },
        { "GZMF",        "Total Mole Fraction" },
        { "GCMPR",       "Hydrocarbon Component Molar Production Rates" },
        { "GCMPT",       "Hydrocarbon Component" },
        { "GCMIR",       "Hydrocarbon Component Molar Injection Rates" },
        { "GCMIT",       "Hydrocarbon Component Molar Injection Totals" },
        { "GHMIR",       "Hydrocarbon Molar Injection Rate" },
        { "GHMIT",       "Hydrocarbon Molar Injection Total" },
        { "GHMPR",       "Hydrocarbon Molar Production Rate" },
        { "GHMPT",       "Hydrocarbon Molar Production Total" },
        { "GCHMR",       "Hydrocarbon Component" },
        { "GCHMT",       "Hydrocarbon Component" },
        { "GCWGPR",      "Hydrocarbon Component Wet Gas Production Rate" },
        { "GCWGPT",      "Hydrocarbon Component Wet Gas Production Total" },
        { "GCWGIR",      "Hydrocarbon Component Wet Gas Injection Rate" },
        { "GCWGIT",      "Hydrocarbon Component Wet Gas Injection Total" },
        { "GCGMR",       "Hydrocarbon component" },
        { "GCGMT",       "Hydrocarbon component" },
        { "GCOMR",       "Hydrocarbon component" },
        { "GCOMT",       "Hydrocarbon component" },
        { "GCNMR",       "Hydrocarbon component molar rates in the NGL phase" },
        { "GCNWR",       "Hydrocarbon component mass rates in the NGL phase" },
        { "GCGMRn",      "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "GCGRn",       "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "GCOMRn",      "Hydrocarbon component" },
        { "GCORn",       "Hydrocarbon component" },
        { "GMUF",        "Make-up fraction" },
        { "GAMR",        "Make-up gas rate" },
        { "GAMT",        "Make-up gas total" },
        { "GGSPR",       "Target sustainable rate for most recent sustainable capacity test for gas" },
        { "GGSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for gas" },
        { "GGSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for gas" },
        { "GGSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for gas" },
        { "GGSTP",       "Test period for the most recent sustainable capacity test for gas" },
        { "GOSPR",       "Target sustainable rate for most recent sustainable capacity test for oil" },
        { "GOSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for oil" },
        { "GOSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for oil" },
        { "GOSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for oil" },
        { "GOSTP",       "Test period for the most recent sustainable capacity test for oil" },
        { "GWSPR",       "Target sustainable rate for most recent sustainable capacity test for water" },
        { "GWSRL",       "Maximum tested rate sustained for the test period during the most recent sustainable capacity test for water" },
        { "GWSRU",       "Minimum tested rate not sustained for the test period during the most recent sustainable capacity test for water" },
        { "GWSSP",       "Period for which target sustainable rate could be maintained for the most recent sustainable capacity test for water" },
        { "GWSTP",       "Test period for the most recent sustainable capacity test for water" },
        { "GGPRG",       "Gas production rate" },
        { "GOPRG",       "Oil production rate" },
        { "GNLPRG",      "NGL production rate" },
        { "GXMFG",       "Liquid mole fraction" },
        { "GYMFG",       "Vapor mole fraction" },
        { "GCOMRG",      "Hydrocarbon component" },
        { "GCGMRG",      "Hydrocarbon component molar rates in the gas phase" },
        { "GCNMRG",      "Hydrocarbon component molar rates in the NGL phase" },
        { "GTPR",        "Tracer Production Rate" },
        { "GTPT",        "Tracer Production Total" },
        { "GTPC",        "Tracer Production Concentration" },
        { "GTIR",        "Tracer Injection Rate" },
        { "GTIT",        "Tracer Injection Total" },
        { "GTIC",        "Tracer Injection Concentration" },
        { "GTMR",        "Traced mass Rate" },
        { "GTMT",        "Traced mass Total" },
        { "GTQR",        "Traced molar Rate" },
        { "GTCM",        "Tracer Carrier molar Rate" },
        { "GTMF",        "Traced molar fraction" },
        { "GTVL",        "Traced liquid volume rate" },
        { "GTVV",        "Traced vapor volume rate" },
        { "GTTL",        "Traced liquid volume total" },
        { "GTTV",        "Traced vapor volume total" },
        { "GTML",        "Traced mass liquid rate" },
        { "GTMV",        "Traced mass vapor rate" },
        { "GTLM",        "Traced mass liquid total" },
        { "GTVM",        "Traced mass vapor total" },
        { "GAPI",        "Oil API" },
        { "GSPR",        "Salt Production Rate" },
        { "GSPT",        "Salt Production Total" },
        { "GSIR",        "Salt Injection Rate" },
        { "GSIT",        "Salt Injection Total" },
        { "GSPC",        "Salt Production Concentration" },
        { "GSIC",        "Salt Injection Concentration" },
        { "WTPRANI",     "Anion Production Rate" },
        { "WTPTANI",     "Anion Production Total" },
        { "WTIRANI",     "Anion Injection Rate" },
        { "WTITANI",     "Anion Injection Total" },
        { "WTPRCAT",     "Cation Production Rate" },
        { "WTPTCAT",     "Cation Production Total" },
        { "WTIRCAT",     "Cation Injection Rate" },
        { "WTITCAT",     "Cation Injection Total" },
        { "GTPCHEA",     "Production Temperature" },
        { "GTICHEA",     "Injection Temperature" },
        { "GTPRHEA",     "Energy flows" },
        { "GTPTHEA",     "Energy Production Total" },
        { "GTIRHEA",     "Energy flows" },
        { "GTITHEA",     "Energy Injection Total" },
        { "GTPR",        "Tracer Production Rate" },
        { "GTPT",        "Tracer Production Total" },
        { "GTPC",        "Tracer Production Concentration" },
        { "GTIR",        "Tracer Injection Rate" },
        { "GTIT",        "Tracer Injection Total" },
        { "GTIC",        "Tracer Injection Concentration" },
        { "GTIRF",       "Tracer Injection Rate" },
        { "GTIRS",       "Tracer Injection Rate" },
        { "GTPRF",       "Tracer Production Rate" },
        { "GTPRS",       "Tracer Production Rate" },
        { "GTITF",       "Tracer Injection Total" },
        { "GTITS",       "Tracer Injection Total" },
        { "GTPTF",       "Tracer Production Total" },
        { "GTPTS",       "Tracer Production Total" },
        { "GTICF",       "Tracer Injection Concentration" },
        { "GTICS",       "Tracer Injection Concentration" },
        { "GTPCF",       "Tracer Production" },
        { "GTPCS",       "Tracer Production" },
        { "GMPR",        "Methane Production Rate" },
        { "GMPT",        "Methane Production Total" },
        { "GMIR",        "Methane Injection Rate" },
        { "GMIT",        "Methane Injection Total" },
        { "GTPRFOA",     "Production Rate" },
        { "GTPTFOA",     "Production Total" },
        { "GTIRFOA",     "Injection Rate" },
        { "GTITFOA",     "Injection Total" },
        { "GSGR",        "Sales Gas Rate" },
        { "GGSR",        "Sales Gas Rate" },
        { "GSGT",        "Sales Gas Total" },
        { "GGST",        "Sales Gas Total" },
        { "GGDC",        "Gas Delivery Capacity" },
        { "GGDCQ",       "Field/Group Gas DCQ" },
        { "GMCPL",       "Group Multi-level Compressor Level" },
        { "GPR",         "Group nodal Pressure in network" },
        { "GPRDC",       "Group Pressure at Delivery Capacity" },
        { "GGCR",        "Gas consumption rate, at and below this group" },
        { "GGCT",        "Gas consumption cumulative total, at and below this group" },
        { "GFGR",        "Fuel Gas rate, at and below this group" },
        { "GFGT",        "Fuel Gas cumulative total, at and below this group" },
        { "GGIMR",       "Gas import rate, at and below this group" },
        { "GGIMT",       "Gas import cumulative total, at and below this group" },
        { "GPRFP",       "Group or node Pressure in network from end of First Pass" },
        { "GGPRNBFP",    "Gas flow rate along Groups or nodes outlet branch in network, from end of First Pass" },
        { "GGLIR",       "Gas Lift Injection Rate" },
        { "GGCV",        "Gas Calorific Value" },
        { "GGQ",         "Gas molar Quality" },
        { "GEPR",        "Energy Production Rate" },
        { "GEPT",        "Energy Production Total" },
        { "GESR",        "Energy Sales Rate" },
        { "GEST",        "Energy Sales Total" },
        { "GEDC",        "Energy Delivery Capacity" },
        { "GEDCQ",       "Energy DCQ" },
        { "GPR",         "Group or node Pressure in the production network" },
        { "GPRG",        "Group or node Pressure in the gas injection network" },
        { "GPRW",        "Group or node Pressure in the water injection network" },
        { "GPRB",        "Pressure drop along the groups or nodes outlet branch in the production network" },
        { "GPRBG",       "Pressure drop along the groups or nodes inlet branch in the gas injection network" },
        { "GPRBW",       "Pressure drop along the groups or nodes inlet branch in the water injection network" },
        { "GALQ",        "ALQ in the groups or nodes outlet branch in the production network" },
        { "GOPRNB",      "Oil flow rate along the groups or nodes outlet branch in the production network" },
        { "GWPRNB",      "Water flow rate along the groups or nodes outlet branch in the production network" },
        { "GGPRNB",      "Gas flow rate along the groups or nodes outlet branch in the production network" },
        { "GLPRNB",      "Liquid flow rate along the groups or nodes outlet branch in the production network" },
        { "GWIRNB",      "Water flow rate along the groups or nodes inlet branch in the water injection network" },
        { "GGIRNB",      "Gas flow rate along the groups or nodes inlet branch in the gas injection network" },
        { "GOMNR",       "Group or node minimum oil rate as specified with GNETDP in the production network" },
        { "GGMNR",       "Group or node minimum gas rate as specified with GNETDP in the production network" },
        { "GWMNR",       "Group or node minimum water rate as specified with GNETDP in the production network" },
        { "GLMNR",       "Group or node minimum liquid rate as specified with GNETDP in the production network" },
        { "GOMXR",       "Group or node maximum oil rate as specified with GNETDP in the production network" },
        { "GGMXR",       "Group or node maximum gas rate as specified with GNETDP in the production network" },
        { "GWMXR",       "Group or node maximum water rate as specified with GNETDP in the production network" },
        { "GLMXR",       "Group or node maximum liquid rate as specified with GNETDP in the production network" },
        { "GMNP",        "Group or node minimum pressure as specified with GNETDP in the production network" },
        { "GMXP",        "Group or node maximum pressure as specified with GNETDP in the production network" },
        { "GPRINC",      "Group or node pressure increment as specified with GNETDP in the production network" },
        { "GPRDEC",      "Group or node pressure decrement as specified with GNETDP in the production network" },
        { "GCPR",        "Polymer Production Rate" },
        { "GCPC",        "Polymer Production Concentration" },
        { "GCPT",        "Polymer Production Total" },
        { "GCIR",        "Polymer Injection Rate" },
        { "GCIC",        "Polymer Injection Concentration" },
        { "GCIT",        "Polymer Injection Total" },
        { "GSPR",        "Salt Production Rate" },
        { "GSPT",        "Salt Production Total" },
        { "GSIR",        "Salt Injection Rate" },
        { "GSIT",        "Salt Injection Total" },
        { "GOPRL",       "Group Oil Production Rate Target" },
        { "GOIRL",       "Group Oil Injection Rate Target" },
        { "GWPRL",       "Group Water Production Rate Target" },
        { "GWIRL",       "Group Water Injection Rate Target" },
        { "GGPRL",       "Group Gas Production Rate Target" },
        { "GGIRL",       "Group Gas Injection Rate Target" },
        { "GLPRL",       "Group Liquid Production Rate Target" },
        { "GVPRL",       "Group reservoir Volume Production Rate Target" },
        { "GVIRL",       "Group reservoir Volume Injection Rate Target" },
        { "GNPR",        "Solvent Production Rate" },
        { "GNPT",        "Solvent Production Total" },
        { "GNIR",        "Solvent Injection Rate" },
        { "GNIT",        "Solvent Injection Total" },
        { "GTPRSUR",     "Production Rate" },
        { "GTPTSUR",     "Production Total" },
        { "GTIRSUR",     "Injection Rate" },
        { "GTITSUR",     "Injection Total" },
        { "GTPRALK",     "Production Rate" },
        { "GTPTALK",     "Production Total" },
        { "GTIRALK",     "Injection Rate" },
        { "GTITALK",     "Injection Total" },
        { "GU",          "User-defined group quantity" },

        { "WOPR",        "Oil Production Rate" },
        { "WOPRA",       "Oil Production Rate above GOC" },
        { "WOPRB",       "Oil Production Rate below GOC" },
        { "WOPTA",       "Oil Production Total above GOC" },
        { "WOPTB",       "Oil Production Total below GOC" },
        { "WOPR1",       "Oil Production Rate above GOC" },
        { "WOPR2",       "Oil Production Rate below GOC" },
        { "WOPT1",       "Oil Production Total above GOC" },
        { "WOPT2",       "Oil Production Total below GOC" },
        { "WOMR",        "Oil Mass Rate" },
        { "WOMT",        "Oil Mass Total" },
        { "WODN",        "Oil Density at Surface Conditions" },
        { "WOPRH",       "Oil Production Rate History" },
        { "WOPRT",       "Oil Production Rate Target/Limit" },
        { "WOPRF",       "Free Oil Production Rate" },
        { "WOPRS",       "Solution Oil Production Rate" },
        { "WOPT",        "Oil Production Total" },
        { "WOPTH",       "Oil Production Total History" },
        { "WOPTF",       "Free Oil Production Total" },
        { "WOPTS",       "Solution Oil Production Total" },
        { "WOIR",        "Oil Injection Rate" },
        { "WOIRH",       "Oil Injection Rate History" },
        { "WOIRT",       "Oil Injection Rate Target/Limit" },
        { "WOIT",        "Oil Injection Total" },
        { "WOITH",       "Oil Injection Total History" },
        { "WOPP",        "Oil Potential Production rate" },
        { "WOPP2",       "Oil Potential Production rate" },
        { "WOPI",        "Oil Potential Injection rate" },
        { "WOPI2",       "Oil Potential Injection rate" },
        { "WOPGR",       "Oil Production Guide Rate" },
        { "WOIGR",       "Oil Injection Guide Rate" },
        { "WOVPR",       "Oil Voidage Production Rate" },
        { "WOVPT",       "Oil Voidage Production Total" },
        { "WOVIR",       "Oil Voidage Injection Rate" },
        { "WOVIT",       "Oil Voidage Injection Total" },
        { "WOnPR",       "nth separator stage oil rate" },
        { "WOnPT",       "nth separator stage oil total" },
        { "WWPR",        "Water Production Rate" },
        { "WWMR",        "Water Mass Rate" },
        { "WWMT",        "Water Mass Total" },
        { "WWPRH",       "Water Production Rate History" },
        { "WWPRT",       "Water Production Rate Target/Limit" },
        { "WWPT",        "Water Production Total" },
        { "WWPTH",       "Water Production Total History" },
        { "WWIR",        "Water Injection Rate" },
        { "WWIRH",       "Water Injection Rate History" },
        { "WWIRT",       "Water Injection Rate Target/Limit" },
        { "WWIT",        "Water Injection Total" },
        { "WWITH",       "Water Injection Total History" },
        { "WWPP",        "Water Potential Production rate" },
        { "WWPP2",       "Water Potential Production rate" },
        { "WWPI",        "Water Potential Injection rate" },
        { "WWIP",        "Water Potential Injection rate" },
        { "WWPI2",       "Water Potential Injection rate" },
        { "WWIP2",       "Water Potential Injection rate" },
        { "WWPGR",       "Water Production Guide Rate" },
        { "WWIGR",       "Water Injection Guide Rate" },
        { "WWVPR",       "Water Voidage Production Rate" },
        { "WWVPT",       "Water Voidage Production Total" },
        { "WWVIR",       "Water Voidage Injection Rate" },
        { "WWVIT",       "Water Voidage Injection Total" },
        { "WWPIR",       "Ratio of produced water to injected water (percentage)" },
        { "WWMPR",       "Water component molar production rate" },
        { "WWMPT",       "Water component molar production total" },
        { "WWMIR",       "Water component molar injection rate" },
        { "WWMIT",       "Water component molar injection total" },
        { "WGPR",        "Gas Production Rate" },
        { "WGPRA",       "Gas Production Rate above" },
        { "WGPRB",       "Gas Production Rate below" },
        { "WGPTA",       "Gas Production Total above" },
        { "WGPTB",       "Gas Production Total below" },
        { "WGPR1",       "Gas Production Rate above GOC" },
        { "WGPR2",       "Gas Production Rate below GOC" },
        { "WGPT1",       "Gas Production Total above GOC" },
        { "WGPT2",       "Gas Production Total below GOC" },
        { "WGMR",        "Gas Mass Rate" },
        { "WGMT",        "Gas Mass Total" },
        { "WGDN",        "Gas Density at Surface Conditions" },
        { "WGPRH",       "Gas Production Rate History" },
        { "WGPRT",       "Gas Production Rate Target/Limit" },
        { "WGPRF",       "Free Gas Production Rate" },
        { "WGPRS",       "Solution Gas Production Rate" },
        { "WGPT",        "Gas Production Total" },
        { "WGPTH",       "Gas Production Total History" },
        { "WGPTF",       "Free Gas Production Total" },
        { "WGPTS",       "Solution Gas Production Total" },
        { "WGIR",        "Gas Injection Rate" },
        { "WGIRH",       "Gas Injection Rate History" },
        { "WGIRT",       "Gas Injection Rate Target/Limit" },
        { "WGIT",        "Gas Injection Total" },
        { "WGITH",       "Gas Injection Total History" },
        { "WGPP",        "Gas Potential Production rate" },
        { "WGPP2",       "Gas Potential Production rate" },
        { "WGPPS",       "Solution" },
        { "WGPPS2",      "Solution" },
        { "WGPPF",       "Free Gas Potential Production rate" },
        { "WGPPF2",      "Free Gas Potential Production rate" },
        { "WGPI",        "Gas Potential Injection rate" },
        { "WGIP",        "Gas Potential Injection rate" },
        { "WGPI2",       "Gas Potential Injection rate" },
        { "WGIP2",       "Gas Potential Injection rate" },
        { "WGPGR",       "Gas Production Guide Rate" },
        { "WGIGR",       "Gas Injection Guide Rate" },
        { "WGLIR",       "Gas Lift Injection Rate" },
        { "WWGPR",       "Wet Gas Production Rate" },
        { "WWGPT",       "Wet Gas Production Total" },
        { "WWGPRH",      "Wet Gas Production Rate History" },
        { "WWGIR",       "Wet Gas Injection Rate" },
        { "WWGIT",       "Wet Gas Injection Total" },
        { "WGnPR",       "nth separator stage gas rate" },
        { "WGnPT",       "nth separator stage gas total" },
        { "WGVPR",       "Gas Voidage Production Rate" },
        { "WGVPT",       "Gas Voidage Production Total" },
        { "WGVIR",       "Gas Voidage Injection Rate" },
        { "WGVIT",       "Gas Voidage Injection Total" },
        { "WGQ",         "Gas Quality" },
        { "WLPR",        "Liquid Production Rate" },
        { "WLPRH",       "Liquid Production Rate History" },
        { "WLPRT",       "Liquid Production Rate Target/Limit" },
        { "WLPT",        "Liquid Production Total" },
        { "WLPTH",       "Liquid Production Total History" },
        { "WVPR",        "Res Volume Production Rate" },
        { "WVPRT",       "Res Volume Production Rate Target/Limit" },
        { "WVPT",        "Res Volume Production Total" },
        { "WVPGR",       "Res Volume Production Guide Rate" },
        { "WVIR",        "Res Volume Injection Rate" },
        { "WVIRT",       "Res Volume Injection Rate Target/Limit" },
        { "WVIT",        "Res Volume Injection Total" },
        { "WWCT",        "Water Cut" },
        { "WWCTH",       "Water Cut History" },
        { "WGOR",        "Gas-Oil Ratio" },
        { "WGORH",       "Gas-Oil Ratio History" },
        { "WOGR",        "Oil-Gas Ratio" },
        { "WOGRH",       "Oil-Gas Ratio History" },
        { "WWGR",        "Water-Gas Ratio" },
        { "WWGRH",       "Water-Gas Ratio History" },
        { "WGLR",        "Gas-Liquid Ratio" },
        { "WGLRH",       "Gas-Liquid Ratio History" },
        { "WBGLR",       "Bottom hole Gas-Liquid Ratio" },
        { "WBHP",        "Bottom Hole Pressure" },
        { "WBHPH",       "Bottom Hole Pressure History," },
        { "WBHPT",       "Bottom Hole Pressure Target/Limit" },
        { "WTHP",        "Tubing Head Pressure" },
        { "WTHPH",       "Tubing Head Pressure History," },
        { "WPI",         "Productivity Index of wells preferred phase" },
        { "WPIO",        "Oil phase PI" },
        { "WPIG",        "Gas phase PI" },
        { "WPIW",        "Water phase PI" },
        { "WPIL",        "Liquid phase PI" },
        { "WBP",         "One-point Pressure Average" },
        { "WBP4",        "Four-point Pressure Average" },
        { "WBP5",        "Five-point Pressure Average" },
        { "WBP9",        "Nine-point Pressure Average" },
        { "WPI1",        "Productivity Index based on the value of WBP" },
        { "WPI4",        "Productivity Index based on the value of WBP4" },
        { "WPI5",        "Productivity Index based on the value of WBP5" },
        { "WPI9",        "Productivity Index based on the value of WBP9" },
        { "WHD",         "Hydraulic head in well based on the reference depth given in HYDRHEAD and the wells reference depth" },
        { "WHDF",        "Hydraulic head in well based on the reference depth given in HYDRHEAD and the wells reference depth calculated at freshwater conditions" },
        { "WSTAT",       "Well State Indicator" },
        { "WMCTL",       "Mode of Control" },
        { "WMCON",       "The number of connections capable of flowing in the well" },
        { "WEPR",        "Energy Production Rate" },
        { "WEPT",        "Energy Production Total" },
        { "WEFF",        "Efficiency Factor" },
        { "WEFFG",       "Product of efficiency factors of the well and all its superior groups" },
        { "WALQ",        "Well Artificial Lift Quantity" },
        { "WMVFP",       "VFP table number used by the well" },
        { "WNLPR",       "NGL Production Rate" },
        { "WNLPT",       "NGL Production Total" },
        { "WNLPRH",      "NGL Production Rate History" },
        { "WNLPTH",      "NGL Production Total History" },
        { "WNLPRT",      "NGL Production Rate Target" },
        { "WAMF",        "Component aqueous mole fraction, from producing completions" },
        { "WXMF",        "Liquid Mole Fraction" },
        { "WYMF",        "Vapor Mole Fraction" },
        { "WXMFn",       "Liquid Mole Fraction for nth separator stage" },
        { "WYMFn",       "Vapor Mole Fraction for nth separator stage" },
        { "WZMF",        "Total Mole Fraction" },
        { "WCMPR",       "Hydrocarbon Component Molar Production Rates" },
        { "WCMPT",       "Hydrocarbon Component" },
        { "WCMIR",       "Hydrocarbon Component Molar Injection Rates" },
        { "WCMIT",       "Hydrocarbon Component Molar Injection Totals" },
        { "WCGIR",       "Hydrocarbon Component Gas Injection Rate" },
        { "WCGPR",       "Hydrocarbon Component Gas Production Rate" },
        { "WCOPR",       "Hydrocarbon Component Oil Production Rate" },
        { "WHMIR",       "Hydrocarbon Molar Injection Rate" },
        { "WHMIT",       "Hydrocarbon Molar Injection Total" },
        { "WHMPR",       "Hydrocarbon Molar Production Rate" },
        { "WHMPT",       "Hydrocarbon Molar Production Total" },
        { "WCHMR",       "Hydrocarbon Component" },
        { "WCHMT",       "Hydrocarbon Component" },
        { "WCWGPR",      "Hydrocarbon Component Wet Gas Production Rate" },
        { "WCWGPT",      "Hydrocarbon Component Wet Gas Production Total" },
        { "WCWGIR",      "Hydrocarbon Component Wet Gas Injection Rate" },
        { "WCWGIT",      "Hydrocarbon Component Wet Gas Injection Total" },
        { "WCGMR",       "Hydrocarbon component" },
        { "WCGMT",       "Hydrocarbon component" },
        { "WCOMR",       "Hydrocarbon component" },
        { "WCOMT",       "Hydrocarbon component" },
        { "WCNMR",       "Hydrocarbon component molar rates in the NGL phase" },
        { "WCNWR",       "Hydrocarbon component mass rates in the NGL phase" },
        { "WCGMRn",      "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "WCGRn",       "Hydrocarbon component molar rates in the gas phase for nth separator stage" },
        { "WCOMRn",      "Hydrocarbon component" },
        { "WCORn",       "Hydrocarbon component" },
        { "WMUF",        "Make-up fraction" },
        { "WTHT",        "Tubing Head Temperature" },
        { "WMMW",        "Mean molecular weight of wellstream" },
        { "WPWE0",       "Well drilled indicator" },
        { "WPWE1",       "Connections opened indicator" },
        { "WPWE2",       "Connections closed indicator" },
        { "WPWE3",       "Connections closed to bottom indicator" },
        { "WPWE4",       "Well stopped indicator" },
        { "WPWE5",       "Injector to producer indicator" },
        { "WPWE6",       "Producer to injector indicator" },
        { "WPWE7",       "Well shut indicator" },
        { "WPWEM",       "WELEVNT output mnemonic" },
        { "WDRPR",       "Well drilling priority" },
        { "WBHWCn",      "Derivative of well BHP with respect to parameter n" },
        { "WGFWCn",      "Derivative of well gas flow rate with respect to parameter n" },
        { "WOFWCn",      "Derivative of well oil flow rate with respect to parameter n" },
        { "WWFWCn",      "Derivative of water flow rate with respect to parameter n" },
        { "WTPR",        "Tracer Production Rate" },
        { "WTPT",        "Tracer Production Total" },
        { "WTPC",        "Tracer Production Concentration" },
        { "WTIR",        "Tracer Injection Rate" },
        { "WTIT",        "Tracer Injection Total" },
        { "WTIC",        "Tracer Injection Concentration" },
        { "WTMR",        "Traced mass Rate" },
        { "WTMT",        "Traced mass Total" },
        { "WTQR",        "Traced molar Rate" },
        { "WTCM",        "Tracer Carrier molar Rate" },
        { "WTMF",        "Traced molar fraction" },
        { "WTVL",        "Traced liquid volume rate" },
        { "WTVV",        "Traced vapor volume rate" },
        { "WTTL",        "Traced liquid volume total" },
        { "WTTV",        "Traced vapor volume total" },
        { "WTML",        "Traced mass liquid rate" },
        { "WTMV",        "Traced mass vapor rate" },
        { "WTLM",        "Traced mass liquid total" },
        { "WTVM",        "Traced mass vapor total" },
        { "WAPI",        "Oil API" },
        { "WSPR",        "Salt Production Rate" },
        { "WSPT",        "Salt Production Total" },
        { "WSIR",        "Salt Injection Rate" },
        { "WSIT",        "Salt Injection Total" },
        { "WSPC",        "Salt Production Concentration" },
        { "WSIC",        "Salt Injection Concentration" },
        { "WTPCHEA",     "Production Temperature" },
        { "WTICHEA",     "Injection Temperature" },
        { "WTPRHEA",     "Energy flows" },
        { "WTPTHEA",     "Energy Production Total" },
        { "WTIRHEA",     "Energy flows" },
        { "WTITHEA",     "Energy Injection Total" },
        { "WTPR",        "Tracer Production Rate" },
        { "WTPT",        "Tracer Production Total" },
        { "WTPC",        "Tracer Production Concentration" },
        { "WTIR",        "Tracer Injection Rate" },
        { "WTIT",        "Tracer Injection Total" },
        { "WTIC",        "Tracer Injection Concentration" },
        { "WTIRF",       "Tracer Injection Rate" },
        { "WTIRS",       "Tracer Injection Rate" },
        { "WTPRF",       "Tracer Production Rate" },
        { "WTPRS",       "Tracer Production Rate" },
        { "WTITF",       "Tracer Injection Total" },
        { "WTITS",       "Tracer Injection Total" },
        { "WTPTF",       "Tracer Production Total" },
        { "WTPTS",       "Tracer Production Total" },
        { "WTICF",       "Tracer Injection Concentration" },
        { "WTICS",       "Tracer Injection Concentration" },
        { "WTPCF",       "Tracer Production" },
        { "WTPCS",       "Tracer Production" },
        { "WMPR",        "Methane Production Rate" },
        { "WMPT",        "Methane Production Total" },
        { "WMIR",        "Methane Injection Rate" },
        { "WMIT",        "Methane Injection Total" },
        { "WTPRFOA",     "Production Rate" },
        { "WTPTFOA",     "Production Total" },
        { "WTIRFOA",     "Injection Rate" },
        { "WTITFOA",     "Injection Total" },
        { "WGDC",        "Gas Delivery Capacity" },
        { "NGOPAS",      "Number of iterations to converge DCQ in first pass" },
        { "WGPRFP",      "Well Gas Production Rate from end of First Pass" },
        { "WTHPFP",      "Well Tubing Head Pressure from end of First Pass" },
        { "WBHPFP",      "Well Bottom Hole Pressure from end of First Pass" },
        { "WGLIR",       "Gas Lift Injection Rate" },
        { "WOGLR",       "Well Oil Gas Lift Ratio" },
        { "WGCV",        "Gas Calorific Value" },
        { "WGQ",         "Gas molar Quality" },
        { "WEPR",        "Energy Production Rate" },
        { "WEPT",        "Energy Production Total" },
        { "WEDC",        "Energy Delivery Capacity" },
        { "WCPR",        "Polymer Production Rate" },
        { "WCPC",        "Polymer Production Concentration" },
        { "WCPT",        "Polymer Production Total" },
        { "WCIR",        "Polymer Injection Rate" },
        { "WCIC",        "Polymer Injection Concentration" },
        { "WCIT",        "Polymer Injection Total" },
        { "WSPR",        "Salt Production Rate" },
        { "WSPT",        "Salt Production Total" },
        { "WSIR",        "Salt Injection Rate" },
        { "WSIT",        "Salt Injection Total" },
        { "WNPR",        "Solvent Production Rate" },
        { "WNPT",        "Solvent Production Total" },
        { "WNIR",        "Solvent Injection Rate" },
        { "WNIT",        "Solvent Injection Total" },
        { "WTPRSUR",     "Production Rate" },
        { "WTPTSUR",     "Production Total" },
        { "WTIRSUR",     "Injection Rate" },
        { "WTITSUR",     "Injection Total" },
        { "WTPRALK",     "Production Rate" },
        { "WTPTALK",     "Production Total" },
        { "WTIRALK",     "Injection Rate" },
        { "WTITALK",     "Injection Total" },
        { "WU",          "User-defined well quantity" },

        { "COFR",        "Oil Flow Rate" },
        { "COFRF",       "Free Oil Flow Rate" },
        { "COFRS",       "Solution oil flow rate" },
        { "COFRU",       "Sum of connection oil flow rates upstream of, and including, this connection" },
        { "COPR",        "Oil Production Rate" },
        { "COPT",        "Oil Production Total" },
        { "COPTF",       "Free Oil Production Total" },
        { "COPTS",       "Solution Oil Production Total" },
        { "COIT",        "Oil Injection Total" },
        { "COPP",        "Oil Potential Production rate" },
        { "COPI",        "Oil Potential Injection rate" },
        { "CWFR",        "Water Flow Rate" },
        { "CWFRU",       "Sum of connection water flow rates upstream of, and including, this connection" },
        { "CWPR",        "Water Production Rate" },
        { "CWPT",        "Water Production Total" },
        { "CWIR",        "Water Injection Rate" },
        { "CWIT",        "Water Injection Total" },
        { "CWPP",        "Water Potential Production rate" },
        { "CWPI",        "Water Potential Injection rate" },
        { "CGFR",        "Gas Flow Rate" },
        { "CGFRF",       "Free Gas Flow Rate" },
        { "CGFRS",       "Solution Gas Flow Rate" },
        { "CGFRU",       "Sum of connection gas flow rates upstream of, and including, this connection" },
        { "CGPR",        "Gas Production Rate " },
        { "CGPT",        "Gas Production Total" },
        { "CGPTF",       "Free Gas Production Total" },
        { "CGPTS",       "Solution Gas Production Total" },
        { "CGIR",        "Gas Injection Rate" },
        { "CGIT",        "Gas Injection Total" },
        { "CGPP",        "Gas Potential Production rate" },
        { "CGPI",        "Gas Potential Injection rate" },
        { "CGQ",         "Gas Quality" },
        { "CLFR",        "Liquid Flow Rate" },
        { "CLPT",        "Liquid Production Total" },
        { "CVFR",        "Reservoir" },
        { "CVPR",        "Res Volume Production Rate" },
        { "CVPT",        "Res Volume Production Total" },
        { "CVIR",        "Res Volume Injection Rate" },
        { "CVIT",        "Res Volume Injection Total" },
        { "CWCT",        "Water Cut" },
        { "CGOR",        "Gas-Oil Ratio" },
        { "COGR",        "Oil-Gas Ratio" },
        { "CWGR",        "Water-Gas Ratio" },
        { "CGLR",        "Gas-Liquid Ratio" },
        { "CPR",         "Connection Pressure" },
        { "CPI",         "Productivity Index of wells preferred phase" },
        { "CTFAC",       "Connection Transmissibility Factor" },
        { "CDBF",        "Blocking factor for generalized pseudo-pressure method" },
        { "CGPPTN",      "Generalized pseudo-pressure table update counter" },
        { "CGPPTS",      "Generalized pseudo-pressure table update status" },
        { "CDSM",        "Current mass of scale deposited" },
        { "CDSML",       "Current mass of scale deposited per unit perforation length" },
        { "CDSF",        "PI multiplicative factor due to scale damage" },
        { "CAMF",        "Component aqueous mole fraction, from producing completions" },
        { "CZMF",        "Total Mole Fraction" },
        { "CKFR",        "Hydrocarbon Component" },
        { "CKFT",        "Hydrocarbon Component" },
        { "CDFAC",       "D-factor for flow dependent skin factor" },
        { "CTFR",        "Tracer Flow Rate" },
        { "CTPR",        "Tracer Production Rate" },
        { "CTPT",        "Tracer Production Total" },
        { "CTPC",        "Tracer Production Concentration" },
        { "CTIR",        "Tracer Injection Rate" },
        { "CTIT",        "Tracer Injection Total" },
        { "CTIC",        "Tracer Injection Concentration" },
        { "CAPI",        "Oil API" },
        { "CSFR",        "Salt Flow Rate" },
        { "CSPR",        "Salt Production Rate" },
        { "CSPT",        "Salt Production Total" },
        { "CSIR",        "Salt Injection Rate" },
        { "CSIT",        "Salt Injection Total" },
        { "CSPC",        "Salt Production Concentration" },
        { "CSIC",        "Salt Injection Concentration" },
        { "CTFRANI",     "Anion Flow Rate" },
        { "CTPTANI",     "Anion Production Total" },
        { "CTITANI",     "Anion Injection Total" },
        { "CTFRCAT",     "Cation Flow Rate" },
        { "CTPTCAT",     "Cation Production Total" },
        { "CTITCAT",     "Cation Injection Total" },
        { "CTFR",        "Tracer Flow Rate" },
        { "CTPR",        "Tracer Production Rate" },
        { "CTPT",        "Tracer Production Total" },
        { "CTPC",        "Tracer Production Concentration" },
        { "CTIR",        "Tracer Injection Rate" },
        { "CTIT",        "Tracer Injection Total" },
        { "CTIC",        "Tracer Injection Concentration" },
        { "CTIRF",       "Tracer Injection Rate" },
        { "CTIRS",       "Tracer Injection Rate" },
        { "CTPRF",       "Tracer Production Rate" },
        { "CTPRS",       "Tracer Production Rate" },
        { "CTITF",       "Tracer Injection Total" },
        { "CTITS",       "Tracer Injection Total" },
        { "CTPTF",       "Tracer Production Total" },
        { "CTPTS",       "Tracer Production Total" },
        { "CTICF",       "Tracer Injection Concentration" },
        { "CTICS",       "Tracer Injection Concentration" },
        { "CTPCF",       "Tracer Production" },
        { "CTPCS",       "Tracer Production" },
        { "CTFRFOA",     "Flow Rate" },
        { "CTPTFOA",     "Production Total" },
        { "CTITFOA",     "Injection Total" },
        { "CRREXCH",     "Exchange flux at current time" },
        { "CRRPROT",     "Connection cumulative water production" },
        { "CRRINJT",     "Connection cumulative water injection" },
        { "CCFR",        "Polymer Flow Rate" },
        { "CCPR",        "Polymer Production Rate" },
        { "CCPC",        "Polymer Production Concentration" },
        { "CCPT",        "Polymer Production Total" },
        { "CCIR",        "Polymer Injection Rate" },
        { "CCIC",        "Polymer Injection Concentration" },
        { "CCIT",        "Polymer Injection Total" },
        { "CSFR",        "Salt Flow Rate" },
        { "CSPR",        "Salt Production Rate" },
        { "CSPT",        "Salt Production Total" },
        { "CSIR",        "Salt Injection Rate" },
        { "CSIT",        "Salt Injection Total" },
        { "CNFR",        "Solvent Flow Rate" },
        { "CNPT",        "Solvent Production Total" },
        { "CNIT",        "Solvent Injection Total" },
        { "CTFRSUR",     "Flow Rate" },
        { "CTPTSUR",     "Production Total" },
        { "CTITSUR",     "Injection Total" },
        { "CTFRALK",     "Flow Rate" },
        { "CTPTALK",     "Production Total" },
        { "CTITALK",     "Injection Total" },
        { "COFRU",       "Sum of connection oil flow rates upstream of, and including, this connection" },
        { "CWFRU",       "Sum of connection water flow rates upstream of, and including, this connection" },
        { "CGFRU",       "Sum of connection gas flow rates upstream of, and including, this connection" },
        { "LCOFRU",      "As COFRU but for local grids" },
        { "LCWFRU",      "As CWFRU but for local grids" },
        { "LCGFRU",      "As CGFRU but for local grids" },
        { "CU",          "User-defined connection quantity" },

        { "COFRL",       "Oil Flow Rate" },
        { "WOFRL",       "Oil Flow Rate" },
        { "COPRL",       "Oil Flow Rate" },
        { "WOPRL",       "Oil Flow Rate" },
        { "COPTL",       "Oil Production Total" },
        { "WOPTL",       "Oil Production Total" },
        { "COITL",       "Oil Injection Total" },
        { "WOITL",       "Oil Injection Total" },
        { "CWFRL",       "Water Flow Rate" },
        { "WWFRL",       "Water Flow Rate" },
        { "CWPRL",       "Water Flow Rate" },
        { "WWPRL",       "Water Flow Rate" },
        { "CWPTL",       "Water Production Total" },
        { "WWPTL",       "Water Production Total" },
        { "CWIRL",       "Water Injection Rate" },
        { "WWIRL",       "Water Injection Rate" },
        { "CWITL",       "Water Injection Total" },
        { "WWITL",       "Water Injection Total" },
        { "CGFRL",       "Gas Flow Rate" },
        { "WGFRL",       "Gas Flow Rate" },
        { "CGPRL",       "Gas Flow Rate" },
        { "WGPRL",       "Gas Flow Rate" },
        { "CGPTL",       "Gas Production Total" },
        { "WGPTL",       "Gas Production Total" },
        { "CGIRL",       "Gas Injection Rate" },
        { "WGIRL",       "Gas Injection Rate" },
        { "CGITL",       "Gas Injection Total" },
        { "WGITL",       "Gas Injection Total" },
        { "CLFRL",       "Liquid Flow Rate" },
        { "WLFRL",       "Liquid Flow Rate" },
        { "CLPTL",       "Liquid Production Total" },
        { "WLPTL",       "Liquid Production Total" },
        { "CVFRL",       "Reservoir" },
        { "WVFRL",       "Res Volume Flow Rate" },
        { "CVPRL",       "Res Volume Production Flow Rate" },
        { "WVPRL",       "Res Volume Production Flow Rate" },
        { "CVIRL",       "Res Volume Injection Flow Rate" },
        { "WVIRL",       "Res Volume Injection Flow Rate" },
        { "CVPTL",       "Res Volume Production Total" },
        { "WVPTL",       "Res Volume Production Total" },
        { "CVITL",       "Res Volume Injection Total" },
        { "WVITL",       "Res Volume Injection Total" },
        { "CWCTL",       "Water Cut" },
        { "WWCTL",       "Water Cut" },
        { "CGORL",       "Gas-Oil Ratio" },
        { "WGORL",       "Gas-Oil Ratio" },
        { "COGRL",       "Oil-Gas Ratio" },
        { "WOGRL",       "Oil-Gas Ratio" },
        { "CWGRL",       "Water-Gas Ratio" },
        { "WWGRL",       "Water-Gas Ratio" },
        { "CGLRL",       "Gas-Liquid Ratio" },
        { "WGLRL",       "Gas-Liquid Ratio" },
        { "CPRL",        "Average Connection Pressure in completion" },
        { "CKFRL",       "Hydrocarbon Component" },
        { "CKFTL",       "Hydrocarbon Component" },

        { "ROFR",        "Inter-region oil flow rate" },
        { "ROFR+",       "Inter-region oil flow rate" },
        { "ROFR-",       "Inter-region oil flow rate" },
        { "ROFT",        "Inter-region oil flow total" },
        { "ROFT+",       "Inter-region oil flow total" },
        { "ROFT-",       "Inter-region oil flow total" },
        { "ROFTL",       "Inter-region oil flow total" },
        { "ROFTG",       "Inter-region oil flow total" },
        { "RGFR",        "Inter-region gas flow rate" },
        { "RGFR+",       "Inter-region gas flow rate" },
        { "RGFR-",       "Inter-region gas flow rate" },
        { "RGFT",        "Inter-region gas flow total)" },
        { "RGFT+",       "Inter-region gas flow total" },
        { "RGFT-",       "Inter-region gas flow total" },
        { "RGFTL",       "Inter-region gas flow total" },
        { "RGFTG",       "Inter-region gas flow total" },
        { "RWFR",        "Inter-region water flow rate" },
        { "RWFR+",       "Inter-region water flow rate" },
        { "RWFR-",       "Inter-region water flow rate" },
        { "RWFT",        "Inter-region water flow total" },
        { "RPR",         "Pressure average value" },
        { "RPRH",        "Pressure average value" },
        { "RPRP",        "Pressure average value" },
        { "RPRGZ",       "P/Z" },
        { "RRS",         "Gas-oil ratio" },
        { "RRV",         "Oil-gas ratio" },
        { "RPPC",        "Initial Contact Corrected Potential" },
        { "RRPV",        "Pore Volume at Reservoir conditions" },
        { "ROPV",        "Pore Volume containing Oil" },
        { "RWPV",        "Pore Volume containing Water" },
        { "RGPV",        "Pore Volume containing Gas" },
        { "RHPV",        "Pore Volume containing Hydrocarbon" },
        { "RRTM",        "Transmissibility Multiplier associated with rock compaction" },
        { "ROE",         "(OIP(initial - OIP(now) / OIP(initial)" },
        { "ROEW",        "Oil Production from Wells / OIP(initial)" },
        { "ROEIW",       "(OIP(initial - OIP(now) / Initial Mobile Oil with respect to Water" },
        { "ROEWW",       "Oil Production from Wells / Initial Mobile Oil with respect to Water" },
        { "ROEIG",       "(OIP(initial - OIP(now) / Initial Mobile Oil with respect to Gas" },
        { "ROEWG",       "Oil Production from Wells / Initial Mobile Oil with respect to Gas" },
        { "RORMR",       "Total stock tank oil produced by rock compaction" },
        { "RORMW",       "Total stock tank oil produced by water influx" },
        { "RORMG",       "Total stock tank oil produced by gas influx" },
        { "RORME",       "Total stock tank oil produced by oil expansion" },
        { "RORMS",       "Total stock tank oil produced by solution gas" },
        { "RORMF",       "Total stock tank oil produced by free gas influx" },
        { "RORMX",       "Total stock tank oil produced by 'traced' water influx" },
        { "RORMY",       "Total stock tank oil produced by other water influx" },
        { "RORFR",       "Fraction of total oil produced by rock compaction" },
        { "RORFW",       "Fraction of total oil produced by water influx" },
        { "RORFG",       "Fraction of total oil produced by gas influx" },
        { "RORFE",       "Fraction of total oil produced by oil expansion" },
        { "RORFS",       "Fraction of total oil produced by solution gas" },
        { "RORFF",       "Fraction of total oil produced by free gas influx" },
        { "RORFX",       "Fraction of total oil produced by 'traced' water influx" },
        { "RORFY",       "Fraction of total oil produced by other water influx" },
        { "RTIPT",       "Tracer In Place" },
        { "RTIPF",       "Tracer In Place" },
        { "RTIPS",       "Tracer In Place" },
        { "RTFTF",       "Tracer inter-region Flow Total" },
        { "RTFTS",       "Tracer inter-region Flow Total" },
        { "RTFTT",       "Tracer inter-region Flow Total" },
        { "RAPI",        "Oil API" },
        { "RSIP",        "Salt In Place" },
        { "RSFT",        "Salt inter-region Flow Total" },
        { "RTIPTHEA",    "Difference in Energy in place between current and initial time" },
        { "RTIPT",       "Tracer In Place" },
        { "RTIPF",       "Tracer In Place" },
        { "RTIPS",       "Tracer In Place" },
        { "RTIP#",       "Tracer In Place in phase # (1,2,3,...)" },
        { "RTFTT",       "Tracer inter-region Flow" },
        { "RTFTF",       "Tracer inter-region Flow" },
        { "RTFTS",       "Tracer inter-region Flow" },
        { "RTFT#",       "Tracer inter-region Flow in phase # (1,2,3,...)" },
        { "RTADS",       "Tracer Adsorption total" },
        { "RTDCY",       "Decayed tracer" },
        { "RCGC",        "Bulk Coal Gas Concentration" },
        { "RCSC",        "Bulk Coal Solvent Concentration" },
        { "RTIPTFOA",    "In Solution" },
        { "RTFTTFOA",    "Inter-region Flow Total" },
        { "RTADSFOA",    "Adsorption total" },
        { "RTDCYFOA",    "Decayed tracer" },
        { "RTMOBFOA",    "Gas mobility factor" },
        { "RCIP",        "Polymer In Solution" },
        { "RCFT",        "Polymer inter-region Flow Total" },
        { "RCAD",        "Polymer Adsorption total" },
        { "RSIP",        "Salt In Place" },
        { "RSFT",        "Salt inter-region Flow Total" },
        { "RNIP",        "Solvent In Place" },
        { "RNFT",        "Solvent inter-region Flow" },
        { "RTIPTSUR",    "In Solution" },
        { "RTFTTSUR",    "Inter-region Flow Total" },
        { "RTADSUR",     "Adsorption total" },
        { "RU",          "User-defined region quantity" },

        { "BPR",         "Oil phase Pressure" },
        { "BPRESSUR",    "Oil phase Pressure" },
        { "BWPR",        "Water phase Pressure" },
        { "BGPR",        "Gas phase Pressure" },
        { "BRS",         "Gas-oil ratio" },
        { "BRV",         "Oil-gas ratio" },
        { "BPBUB",       "Bubble point pressure" },
        { "BPDEW",       "Dew point pressure" },
        { "BRSSAT",      "Saturated gas-oil ratio" },
        { "BRVSAT",      "Saturated oil-gas ratio" },
        { "BSTATE",      "Gas-oil state indicator" },
        { "BPPC",        "Initial Contact Corrected Potential" },
        { "BOKR",        "Oil relative permeability" },
        { "BWKR",        "Water relative permeability" },
        { "BGKR",        "Gas relative permeability" },
        { "BKRO",        "Oil relative permeability" },
        { "BKROG",       "Two-phase oil relative permeability to gas" },
        { "BKROW",       "Two-phase oil relative permeability to water" },
        { "BKRG",        "Gas relative permeability" },
        { "BKRGO",       "Two-phase gas relative permeability to oil " },
        { "BKRGW",       "Two-phase gas relative permeability to water" },
        { "BKRW",        "Water relative permeability" },
        { "BKRWG",       "Two-phase water relative permeability to gas" },
        { "BKRWO",       "Two-phase water relative permeability to oil" },
        { "BRK",         "Water relative permeability reduction factor due to polymer" },
        { "BEWKR",       "Water effective relative permeability due to polymer" },
        { "BWPC",        "Water-Oil capillary pressure" },
        { "BGPC",        "Gas-Oil capillary pressure" },
        { "BPCO",        "Oil Capillary Pressures" },
        { "BPCG",        "Gas Capillary Pressures" },
        { "BPCW",        "Water Capillary Pressures" },
        { "BGTRP",       "Trapped gas saturation" },
        { "BGTPD",       "Dynamic trapped gas saturation" },
        { "BGSHY",       "Departure saturation from drainage to imbibition for gas capillary pressure hysteresis" },
        { "BGSTRP",      "Trapped gas critical saturation for gas capillary pressure hysteresis" },
        { "BWSHY",       "Departure saturation from drainage to imbibition for water capillary pressure hysteresis" },
        { "BWSMA",       "Maximum wetting saturation for water capillary pressure hysteresis" },
        { "BMLSC",       "Hydrocarbon molar density" },
        { "BMLST",       "Total hydrocarbon molar density" },
        { "BMWAT",       "Water molar density" },
        { "BROMLS",      "Residual oil moles/ reservoir volume" },
        { "BJV",         "In" },
        { "BVMF",        "Vapor mole fraction" },
        { "BPSAT",       "Saturation Pressures" },
        { "BAMF",        "Component aqueous mole fraction" },
        { "BXMF",        "Liquid hydrocarbon component mole fraction" },
        { "BYMF",        "Vapor hydrocarbon component mole fraction / vapor steam" },
        { "BSMF",        "CO2STORE with SOLID option only Solid hydrocarbon component mole fraction" },
        { "BSTEN",       "Surface Tension" },
        { "BFMISC",      "Miscibility Factor" },
        { "BREAC",       "Reaction rate. The reaction number is given as a component index" },
        { "BHD",         "Hydraulic head" },
        { "BHDF",        "Hydraulic head at fresh water conditions" },
        { "BPR_X",       "Pressure interpolated at a defined coordinate" },
        { "BHD_X",       "Hydraulic head interpolated at a defined coordinate" },
        { "BHDF_X",      "Hydraulic head at fresh water conditions interpolated at a defined coordinate" },
        { "BSCN_X",      "Brine concentration interpolated at a defined coordinate" },
        { "BCTRA_X",     "Tracer concentration interpolated at a defined coordinate" },
        { "LBPR_X",      "Pressure interpolated at a defined coordinate within a local grid" },
        { "LBHD_X",      "Hydraulic head interpolated at a defined coordinate within a local grid" },
        { "LBHDF_X",     "Hydraulic head at freshwater conditions interpolated at a defined coordinate within a local grid" },
        { "LBSCN_X",     "Brine concentration interpolated at a defined coordinate within a local grid" },
        { "LBCTRA_X",    "Tracer concentration interpolated at a defined coordinate within a local grid" },
        { "BOKRX",       "Oil relative permeability in the X direction" },
        { "BOKRX",       "- Oil relative permeability in the -X direction" },
        { "BOKRY",       "Oil relative permeability in the Y direction" },
        { "BOKRY",       "- Oil relative permeability in the -Y direction" },
        { "BOKRZ",       "Oil relative permeability in the Z direction" },
        { "BOKRZ",       "- Oil relative permeability in the -Z direction" },
        { "BWKRX",       "Water relative permeability in the X direction" },
        { "BWKRX",       "- Water relative permeability in the -X direction" },
        { "BWKRY",       "Water relative permeability in the Y direction" },
        { "BWKRY",       "- Water relative permeability in the -Y direction" },
        { "BWKRZ",       "Water relative permeability in the Z direction" },
        { "BWKRZ",       "- Water relative permeability in the -Z direction" },
        { "BGKRX",       "Gas relative permeability in the X direction" },
        { "BGKRX",       "- Gas relative permeability in the -X direction" },
        { "BGKRY",       "Gas relative permeability in the Y direction" },
        { "BGKRY",       "- Gas relative permeability in the -Y direction" },
        { "BGKRZ",       "Gas relative permeability in the Z direction" },
        { "BGKRZ",       "- Gas relative permeability in the -Z direction" },
        { "BOKRI",       "Oil relative permeability in the I direction" },
        { "BOKRI",       "- Oil relative permeability in the -I direction" },
        { "BOKRJ",       "Oil relative permeability in the J direction" },
        { "BOKRJ",       "- Oil relative permeability in the -J direction" },
        { "BOKRK",       "Oil relative permeability in the K direction" },
        { "BOKRK",       "- Oil relative permeability in the -K direction" },
        { "BWKRI",       "Water relative permeability in the I direction" },
        { "BWKRI",       "- Water relative permeability in the -I direction" },
        { "BWKRJ",       "Water relative permeability in the J direction" },
        { "BWKRJ",       "- Water relative permeability in the -J direction" },
        { "BWKRK",       "Water relative permeability in the K direction" },
        { "BWKRK",       "- Water relative permeability in the -K direction" },
        { "BGKRI",       "Gas relative permeability in the I direction" },
        { "BGKRI",       "- Gas relative permeability in the -I direction" },
        { "BGKRJ",       "Gas relative permeability in the J direction" },
        { "BGKRJ",       "- Gas relative permeability in the -J direction" },
        { "BGKRK",       "Gas relative permeability in the K direction" },
        { "BGKRK",       "- Gas relative permeability in the -K direction" },
        { "BOKRR",       "Oil relative permeability in the R" },
        { "BOKRR",       "- Oil relative permeability in the -R" },
        { "BOKRT",       "Oil relative permeability in the T" },
        { "BOKRT",       "- Oil relative permeability in the -T" },
        { "BWKRR",       "Water relative permeability in the R" },
        { "BWKRR",       "- Water relative permeability in the -R" },
        { "BWKRT",       "Water relative permeability in the T" },
        { "BWKRT",       "- Water relative permeability in the -T" },
        { "BGKRR",       "Gas relative permeability in the R" },
        { "BGKRR",       "- Gas relative permeability in the -R" },
        { "BGKRT",       "Gas relative permeability in the T" },
        { "BGKRT",       "- Gas relative permeability in the -T" },
        { "BRPV",        "Pore Volume at Reservoir conditions" },
        { "BPORV",       "Cell Pore Volumes at Reference conditions" },
        { "BOPV",        "Pore Volume containing Oil" },
        { "BWPV",        "Pore Volume containing Water" },
        { "BGPV",        "Pore Volume containing Gas" },
        { "BHPV",        "Pore Volume containing Hydrocarbon" },
        { "BRTM",        "Transmissibility Multiplier associated with rock compaction" },
        { "BPERMMOD",    "Transmissibility Multiplier associated with rock compaction" },
        { "BPERMMDX",    "Directional Transmissibility Multipliers in the X direction, associated with rock compaction" },
        { "BPERMMDY",    "Directional Transmissibility Multipliers in the Y direction, associated with rock compaction" },
        { "BPERMMDZ",    "Directional Transmissibility Multipliers in the Z direction, associated with rock compaction" },
        { "BPORVMOD",    "Pore Volume Multiplier associated with rock compaction" },
        { "BSIGMMOD",    "Dual Porosity Sigma Multiplier associated with rock compaction" },
        { "BTCNF",       "Tracer Concentration" },
        { "BTCNS",       "Tracer Concentration" },
        { "BTCN",        "Tracer Concentration" },
        { "BTIPT",       "Tracer In Place" },
        { "BTIPF",       "Tracer In Place" },
        { "BTIPS",       "Tracer In Place" },
        { "BAPI",        "Oil API" },
        { "BSCN",        "Salt Cell Concentration" },
        { "BSIP",        "Salt In Place" },
        { "BEWV_SAL",    "Effective water viscosity due to salt concentration" },
        { "BTCNFANI",    "Anion Flowing Concentration" },
        { "BTCNFCAT",    "Cation Flowing Concentration" },
        { "BTRADCAT",    "Cation Rock Associated Concentration" },
        { "BTSADCAT",    "Cation Surfactant Associated Concentration" },
        { "BESALSUR",    "Effective Salinity with respect to Surfactant" },
        { "BESALPLY",    "Effective Salinity with respect to Polymer" },
        { "BTCNFHEA",    "Block Temperature" },
        { "BTIPTHEA",    "Difference in Energy in place between current and initial time" },
        { "BTCNF",       "Tracer Concentration" },
        { "BTCNS",       "Tracer Concentration" },
        { "BTCN#",       "Tracer concentration in phase # (1,2,3,...)" },
        { "BTIPT",       "Tracer In Place" },
        { "BTIPF",       "Tracer In Place" },
        { "BTIPS",       "Tracer In Place" },
        { "BTIP#",       "Tracer In Place in phase # (1,2,3,...)" },
        { "BTADS",       "Tracer Adsorption" },
        { "BTDCY",       "Decayed tracer" },
        { "BCGC",        "Bulk Coal Gas Concentration" },
        { "BCSC",        "Bulk Coal Solvent Concentration" },
        { "BTCNFFOA",    "Concentration" },
        { "BFOAM",       "Surfactant concentration" },
        { "BTCNMFOA",    "Capillary number" },
        { "BFOAMCNM",    "Capillary number" },
        { "BTIPTFOA",    "In Solution" },
        { "BTADSFOA",    "Adsorption" },
        { "BTDCYFOA",    "Decayed tracer" },
        { "BTMOBFOA",    "Gas mobility factor" },
        { "BFOAMMOB",    "Gas mobility factor" },
        { "BTHLFFOA",    "Decay Half life" },
        { "BGI",         "Block Gi value" },
        { "BCCN",        "Polymer Concentration" },
        { "BCIP",        "Polymer In Solution" },
        { "BEPVIS",      "Effective polymer solution viscosity" },
        { "BVPOLY",      "Effective polymer solution viscosity" },
        { "BEMVIS",      "Effective mixture" },
        { "BEWV_POL",    "Effective water viscosity" },
        { "BCAD",        "Polymer Adsorption concentration" },
        { "BCDCS",       "Polymer thermal degradation - total mass degraded in previous timestep" },
        { "BCDCR",       "Polymer thermal degradation - total degradation rate" },
        { "BCDCP",       "Polymer thermal degradation solution degradation rate" },
        { "BCDCA",       "Polymer thermal degradation adsorbed degradation rate" },
        { "BCABnnn",     "Adsorbed polymer by highest temperature band at which RRF was calculated" },
        { "BSCN",        "Salt Cell Concentration" },
        { "BSIP",        "Salt In Place" },
        { "BFLOW0I",     "Inter-block water flow rate in the positive I direction multiplied by the corresponding shear multiplier" },
        { "BFLOW0J",     "Inter-block water flow rate in the positive J direction multiplied by the corresponding shear multiplier" },
        { "BFLOW0K",     "Inter-block water flow rate in the positive K direction multiplied by the corresponding shear multiplier" },
        { "BVELW0I",     "Water velocity in the positive I direction multiplied by the corresponding shear multiplier" },
        { "BVELW0J",     "Water velocity in the positive J direction multiplied by the corresponding shear multiplier" },
        { "BVELW0K",     "Water velocity in the positive K direction multiplied by the corresponding shear multiplier" },
        { "BPSHLZI",     "Viscosity multiplier due to sheared water flow in the positive I direction" },
        { "BPSHLZJ",     "Viscosity multiplier due to sheared water flow in the positive J direction" },
        { "BPSHLZK",     "Viscosity multiplier due to sheared water flow in the positive K direction" },
        { "BSRTW0I",     "Water shear rate in the positive I direction prior to shear effects" },
        { "BSRTW0J",     "Water shear rate in the positive J direction prior to shear effects" },
        { "BSRTW0K",     "Water shear rate in the positive K direction prior to shear effects" },
        { "BSRTWI",      "Water shear rate in the positive I direction following shear effects" },
        { "BSRTWJ",      "Water shear rate in the positive J direction following shear effects" },
        { "BSRTWK",      "Water shear rate in the positive K direction following shear effects" },
        { "BSHWVISI",    "Shear viscosity of the water/polymer solution due to shear thinning/thickening in the positive I direction" },
        { "BSHWVISJ",    "Shear viscosity of the water/polymer solution due to shear thinning/thickening in the positive J direction" },
        { "BSHWVISK",    "Shear viscosity of the water/polymer solution due to shear thinning/thickening in the positive K direction" },
        { "BNSAT",       "Solvent SATuration" },
        { "BNIP",        "Solvent In Place" },
        { "BNKR",        "Solvent relative permeability" },
        { "BTCNFSUR",    "Concentration" },
        { "BSURF",       "Concentration in solution" },
        { "BTIPTSUR",    "In Solution" },
        { "BTADSUR",     "Adsorption" },
        { "BTCASUR",     "Log" },
        { "BSURFCNM",    "Log" },
        { "BTSTSUR",     "Surface tension" },
        { "BSURFST",     "Surface tension" },
        { "BEWV_SUR",    "Effective water viscosity due to surfactant concentration" },
        { "BESVIS",      "Effective water viscosity due to surfactant concentration" },
        { "BTCNFALK",    "Concentration" },
        { "BTADSALK",    "Adsorption" },
        { "BTSTMALK",    "Surface tension multiplier" },
        { "BTSADALK",    "Surfactant adsorption multiplier" },
        { "BTPADALK",    "Polymer adsorption multiplier" },
        { "BKRGOE",      "Equivalent relative permeability to gas for gas-oil system" },
        { "BKRGWE",      "Equivalent relative permeability to gas for gas-water system" },
        { "BKRWGE",      "Equivalent relative permeability to water for water-gas system" },
        { "BKROWT",      "Opposite saturation direction turning point relative permeability to oil for oil-water system" },
        { "BKRWOT",      "Opposite saturation direction turning point relative permeability to water for water-oil system" },
        { "BKROGT",      "Opposite saturation direction turning point relative permeability to oil for oil-gas system" },
        { "BKRGOT",      "Opposite saturation direction turning point relative permeability to gas for gas-oil system" },
        { "BKRGWT",      "Opposite saturation direction turning point relative permeability to gas for gas-water system" },
        { "BKRWGT",      "Opposite saturation direction turning point relative permeability to water for water-gas system" },
        { "BIFTOW",      "Oil-water interfacial tension" },
        { "BIFTWO",      "Water-oil interfacial tension" },
        { "BIFTOG",      "Oil-gas interfacial tension" },
        { "BIFTGO",      "Gas-oil interfacial tension" },
        { "BIFTGW",      "Gas-water interfacial tension" },
        { "BIFTWG",      "Water-gas interfacial tension" },
        { "BPCOWR",      "Representative oil-water capillary pressure" },
        { "BPCWOR",      "Representative water-oil capillary pressure" },
        { "BPCOGR",      "Representative oil-gas capillary pressure" },
        { "BPCGOR",      "Representative gas-oil capillary pressure" },
        { "BPCGWR",      "Representative gas-water capillary pressure" },
        { "BPCWGR",      "Representative water-gas capillary pressure" },

        { "SOFR",        "Segment Oil Flow Rate" },
        { "SOFRF",       "Segment Free Oil Flow Rate" },
        { "SOFRS",       "Segment Solution Oil Flow Rate" },
        { "SWFR",        "Segment Water Flow Rate" },
        { "SGFR",        "Segment Gas Flow Rate" },
        { "SGFRF",       "Segment Free Gas Flow Rate" },
        { "SGFRS",       "Segment Solution Gas Flow Rate" },
        { "SKFR",        "Segment Component Flow Rate" },
        { "SCWGFR",      "Segment Component Flow Rate as Wet Gas" },
        { "SHFR",        "Segment Enthalpy Flow Rate" },
        { "SWCT",        "Segment Water Cut" },
        { "SGOR",        "Segment Gas Oil Ratio" },
        { "SOGR",        "Segment Oil Gas Ratio" },
        { "SWGR",        "Segment Water Gas Ratio" },
        { "SPR",         "Segment Pressure" },
        { "SPRD",        "Segment Pressure Drop" },
        { "SPRDF",       "Segment Pressure Drop component due to Friction" },
        { "SPRDH",       "Segment Pressure Drop component due to Hydrostatic head" },
        { "SPRDA",       "Segment Pressure drop due to Acceleration head" },
        { "SPRDM",       "Segment frictional Pressure Drop Multiplier" },
        { "SPPOW",       "Working power of a pull through pump" },
        { "SOFV",        "Segment Oil Flow Velocity" },
        { "SWFV",        "Segment Water Flow Velocity" },
        { "SGFV",        "Segment Gas Flow Velocity" },
        { "SOHF",        "Segment Oil Holdup Fraction" },
        { "SWHF",        "Segment Water Holdup Fraction" },
        { "SGHF",        "Segment Gas Holdup Fraction" },
        { "SDENM",       "Segment fluid mixture density" },
        { "SOVIS",       "Segment oil viscosity" },
        { "SWVIS",       "Segment water viscosity" },
        { "SGVIS",       "Segment gas viscosity" },
        { "SEMVIS",      "Segment effective mixture viscosity" },
        { "SGLPP",       "Segment Gas-Liquid Profile Parameter, C0" },
        { "SGLVD",       "Segment Gas-Liquid Drift Velocity, Vd" },
        { "SOWPP",       "Segment Oil-Water Profile Parameter, C0" },
        { "SOWVD",       "Segment Oil-Water Drift Velocity, Vd" },
        { "SOIMR",       "Segment Oil Import Rate" },
        { "SGIMR",       "Segment Gas Import Rate" },
        { "SWIMR",       "Segment Water Import Rate" },
        { "SHIMR",       "Segment Enthalpy Import Rate" },
        { "SORMR",       "Segment Oil Removal Rate" },
        { "SGRMR",       "Segment Gas Removal Rate" },
        { "SWRMR",       "Segment Water Removal Rate" },
        { "SHRMR",       "Segment Enthalpy Removal Rate" },
        { "SOIMT",       "Segment Oil Import Total" },
        { "SGIMT",       "Segment Gas Import Total" },
        { "SWIMT",       "Segment Water Import Total" },
        { "SHIMT",       "Segment Enthalpy Import Total" },
        { "SORMT",       "Segment Oil Removal Total" },
        { "SGRMT",       "Segment Gas Removal Total" },
        { "SWRMT",       "Segment Water Removal Total" },
        { "SHRMT",       "Segment Enthalpy Removal Total" },
        { "SAPI",        "Segment API value" },
        { "SCFR",        "Segment polymer flow rate" },
        { "SCCN",        "Segment polymer concentration" },
        { "SSFR",        "Segment brine flow rate" },
        { "SSCN",        "Segment brine concentration" },
        { "STFR",        "Segment tracer flow rate" },
        { "STFC",        "Segment tracer concentration" },
        { "SFD",         "Segment diameter for Karst Conduit Calcite Dissolution" },
        { "SPSAT",       "Segment Psat" },
        { "STEM",        "Segment Temperature" },
        { "SENE",        "Segment Energy Density" },
        { "SSQU",        "Segment Steam Quality" },
        { "SCVPR",       "Segment Calorific Value Production Rate" },
        { "SGQ",         "Segment Gas Quality" },
        { "SCSA",        "Segment Cross Sectional Area" },
        { "SSTR",        "Strength of ICD on segment" },
        { "SFOPN",       "Setting of segment" },
        { "SALQ",        "Artificial lift quantity for segment" },
        { "SRRQR",       "Reach flow at current time" },
        { "SRRQT",       "Reach cumulative flow" },
        { "SRBQR",       "Branch flow at current time" },
        { "SRBQT",       "Branch cumulative flow" },
        { "SRTQR",       "River total flow at current time" },
        { "SRTQT",       "River total cumulative flow" },
        { "SRRFLOW",     "Reach flux through cross-sectional area at current time" },
        { "SRRAREA",     "Reach area at current time" },
        { "SRRDEPTH",    "Reach depth at current time" },
        { "SRREXCH",     "Exchange flux at current time" },
        { "SRRFRODE",    "Reach Froude number at current time" },
        { "SRRHEAD",     "Reach hydraulic head at current time" },
        { "SRTFR",       "Reach tracer flow rate" },
        { "SRTFC",       "Reach tracer concentration" },
        { "SRSFR",       "Reach brine flow rate through connections" },
        { "SRSFC",       "Reach brine concentration" },
        { "SU",          "User-defined segment quantity" },

        { "AAQR",        "Aquifer influx rate" },
        { "ALQR",        "Aquifer influx rate" },
        { "AAQT",        "Cumulative aquifer influx" },
        { "ALQT",        "Cumulative aquifer influx" },
        { "AAQRG",       "Aquifer influx rate" },
        { "ALQRG",       "Aquifer influx rate" },
        { "AAQTG",       "Cumulative aquifer influx" },
        { "ALQTG",       "Cumulative aquifer influx" },
        { "AACMR",       "Aquifer component molar influx rate" },
        { "AACMT",       "Aquifer component molar influx totals" },
        { "AAQP",        "Aquifer pressure" },
        { "AAQER",       "Aquifer thermal energy influx rate" },
        { "AAQET",       "Cumulative aquifer thermal energy influx" },
        { "AAQTEMP",     "Aquifer temperature" },
        { "AAQENTH",     "Aquifer molar enthalpy" },
        { "AAQTD",       "Aquifer dimensionless time" },
        { "AAQPD",       "Aquifer dimensionless pressure" },
        { "ANQR",        "Aquifer influx rate" },
        { "ANQT",        "Cumulative aquifer influx" },
        { "ANQP",        "Aquifer pressure" }
    };
}
